/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import java.sql.*;
import java.util.*;

/**
 * ConnectionServer:
 *
 * A base class for connecting/disconnecting to/from a JDBC source and
 * serving Connection and Statement objects.
 *
 * The class accepts LostConnectionListeners which are notified on a
 * lost connection.
 *
 * The class is intended to centralize the Connection and Statement
 * resources which are to be shared across an application.
 * 
 * This object is allows for the requesting and releasing of Statement
 * objects in a controlled manner, hopefully minimizing the risk of
 * memory and resource shortages.
 *
 * The ConnectionServer is a "monitor", controlling access to JDBC
 * resources.
 *
 * The maxStatements member can be set to control the number of
 * Statement objects returned via getStatement(). The releaseStatement()
 * method indicates that a Statement is no longer in use.
 * 
 * This interface (getStatement() and releaseStatement()) is a voluntary
 * interface and need not be adhered to. Setting maxStatements to 0 or -ve
 * will disable this interface.
 *
 * The releaseStatement() only indicates that a Statement is no longer
 * needed - it does not close() or free its resources in any way.
 *
 * A caller of getStatement() will block until a Statement is available.
 * Calling releaseStatement() will result in a notifyAll() call,
 * unblocking the waiting threads. There is the potential for dead-locks
 * when using this resource without care.
 * 
 */


public class ConnectionServer {

    private Connection conn = null;

    private Hashtable lostConnectionListeners = null;

    private String userName = null;
    private String passWord = null;
    private String driverName = null;
    private String serverAddress = null;

    private int statementsInUse = 0;
    private int maxStatements = 1;

    private boolean driverLoaded = false;




    /**
     * The Properties collection is expected to contain the following
     * properties:
     *     UserName, Password, Driver, DatabaseServerURL.
     */

    public ConnectionServer(Properties p) {
        if (p != null) {
            userName = p.getProperty("UserName");
            passWord = p.getProperty("Password");
            driverName = p.getProperty("Driver");
            serverAddress = p.getProperty("DatabaseServerURL");
        }
    }




    /**
     * The connectToServer() call will attempt to load the driver and connect
     * to the specified server. It is used to re-connect after a
     * connection failure as well.
     * @exception SQLException re-thrown from Connection::getConnection().
     */

    public synchronized boolean connectToServer() throws SQLException {
        boolean rc = true;
        if (userName == null || passWord == null
            || driverName == null || serverAddress == null) {
                return false;
        }
        // can not use isValid() here as we do not want to warn all
        // LostConnectionListeners at this stage.
        if (conn != null && conn.isClosed() == false) {
            return true;
        }
        if (!driverLoaded) {
            // load the driver
            try {
                Class.forName(driverName);
                driverLoaded = true;
            } catch (ClassNotFoundException e) {
                rc = false;
            }

        }
        if (driverLoaded) {
            Properties props = new Properties();
            props.put("user", userName);
            props.put("password", passWord);
            conn = DriverManager.getConnection(serverAddress, props);
            // reset the number of Statements in use.
            statementsInUse = 0;
            // let everyone know that the resource is available.
            notifyAll();
        }
        return rc;
    } // end of connectToServer().




    /**
     * The disconnectFromServer() call will close the connection.
     * @exception SQLException re-thrown from Connection::close().
     */
    public synchronized boolean disconnectFromServer() throws SQLException {
        boolean rc = true;
        if (conn != null) {
            conn.close();
        }
        conn = null;
        return rc;
    }




    /**
     *  Connection validity is determined by the Connection status.
     */

    // public synchronized boolean isValid() {
    public boolean isValid() {
        boolean rc = true;
        if (conn == null) {
            rc = false;
        } else {
            try {
                if (conn.isClosed()) {
                    fireLostConnection();
                    // connection may be re-established by
                    // lostConnectionListener.
                    if (conn != null && conn.isClosed()) {
                        rc = false;
                    }
                }
            } catch (SQLException e) {
                rc = false;
            }
        }
        return rc;
    }




    public Connection getConnection() {
        Connection rc = null;
        if (isValid()) {
            rc = conn;
        } else {
            rc = null;
        }
        return rc;
    }




    /**
     * Create and return a Statement object from the Connection.
     * @exception SQLException re-thrown from Connection::createStatement().
     */

    // NK public synchronized Statement getStatement() throws SQLException {
    public Statement getStatement() throws SQLException {
        return getStatement(false);
    }

    // NK public synchronized Statement requestStatement() throws SQLException {
    public Statement requestStatement() throws SQLException {
        return getStatement(true);
    }

    private Statement getStatement(boolean waitForResource)
            throws SQLException {
        Statement rc = null;

        // if controlling the Statement resource, wait() untill
        // a Statement is available.
        if (maxStatements > 0) {
            if (waitForResource) {
                while (statementsInUse >= maxStatements) {
                    try {
                        wait();
                    } catch (InterruptedException e) {}
                }
            } else {
                if (statementsInUse >= maxStatements) {
                    return rc;
                }
            }
        }
        // if the connection has not been made, connect it now.
        // if connected, this call simply returns.
        if (isValid()) {
            rc = conn.createStatement();
        }
        if (rc != null) {
            statementsInUse++;
        }
        return rc;
    }




    public synchronized int getNumStatementsInUse() {
        return statementsInUse;
    }




    public synchronized void setMaxStatements(int ms) {
        maxStatements = ms;
        notifyAll();
    }




    public synchronized void releaseStatement(Statement s) 
            throws SQLException {
        if (s == null) {
            return;
        }
        SQLException ex = null;
        try {
            s.close();
        } catch (SQLException e) {
            ex = e;
        } finally {
            if (statementsInUse > 0) {
                statementsInUse--;
                // let everyone know of the new resource.
                notifyAll();
            }
        }
        if (ex != null) {
            throw ex;
        }
        return;
    }




    public void addLostConnectionListener(LostConnectionListener l) {
        if (lostConnectionListeners == null) {
            lostConnectionListeners = new Hashtable();
        }
        lostConnectionListeners.put(l, l);
    }




    public void removeLostConnectionListener(LostConnectionListener l) {
        if (lostConnectionListeners != null ) {
            lostConnectionListeners.remove(l);
        }
    }




    public void fireLostConnection() {
        if (lostConnectionListeners != null) {
            Enumeration en = lostConnectionListeners.elements();
            while (en.hasMoreElements())  {
                LostConnectionListener l 
                    = (LostConnectionListener)en.nextElement();
                l.connectionLost();
            }
        }
    }

} // end of class ConnectionServer.

// end of file ConnectionServer.java
