/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import java.sql.*;
import java.util.*;
import java.text.*;

/**
 * DFSLogConnectionServer:
 *
 * A class for connecting to, querying and inserting messages into the
 * the dfslog database.
 * 
 * The class receives most connection and address information by way of a
 * Properties object, which is passed to the super() constructor.
 * 
 */

public class DFSLogConnectionServer extends ConnectionServer {

    private String mostRecentMessage = null;
    private String mostRecentMessageWorkstation = null;
    private long mostRecentMessageId = 0;
    private String mostRecentMessageComponent = null;
    private String mostRecentMessageLabel = null;

    static final String[] priorities = 
        { "Emergency", "Alert", "Critical", "Error",
        "Warning", "Notice", "Info", "Debug" };

    private String recentMessageSelect = null;
    private static final String lowLevelError = "JZ006";
    private static final int errorThreshold = 3;


    public DFSLogConnectionServer(Properties p) {
        super(p);
        if (p != null) {
            recentMessageSelect = p.getProperty("AlarmLineMessageSelect");
        }
    }




    /**
     * Read the most recent, high priority message from the database or
     * from the the cached value (depending on readFromDb parameter).
     * @exception: throws SQLException on errors.
     */

    public String getMostRecentMessage(boolean readFromDb) 
            throws SQLException {
        String rc = null;
	boolean endure = true;
	int counter = 0;

        if (readFromDb == false) {
            rc = mostRecentMessage;
        } else if (recentMessageSelect != null) {
            ResultSet rs;
            ResultSetMetaData rsmd;
            Statement stmt = null;
            SQLException ex = null;
	    while(endure) { 
		try {
		    stmt = getStatement();
		    if (stmt != null) {
			boolean results = stmt.execute(recentMessageSelect);
			if (results) {
			    rs = stmt.getResultSet();
			    rsmd = rs.getMetaData();
			    StringBuffer rcb;
			    int numColumns = rsmd.getColumnCount();
			    if (rs.next()) {
				rcb = new StringBuffer("");
				for (int i = 1; i <= numColumns; i++) {
				    String cn = rsmd.getColumnName(i);
				    String cv = rs.getString(i);
				    if (cn.equals("msg_id")) {
					mostRecentMessageId = rs.getLong(i);
				    } else if (cn.equals("wsid")) {
					mostRecentMessageWorkstation = cv;
				    } else if (cn.equals("compid")) {
					mostRecentMessageComponent = cv;
				    } else if (cn.equals("labelid")) {
					mostRecentMessageLabel = cv;
				    }
				    if (cn.equals("msg_id") == false) {
					rcb.append(" " + cv);
				    }
				}
				rc = rcb.toString();
			    }
			}
		    }
		} catch (SQLException e) {
		    ex = e;
		    System.err.println("DFSLOG STACK TRACE ***********");
		    e.printStackTrace();
		} finally {
		    if (stmt != null) {
			releaseStatement(stmt);
			// set stmt to null so we do not release it twice.
			stmt = null;
		    }
		}
		if (ex != null) {
		    if ( ( ex.toString().indexOf(lowLevelError) != -1) &&
			 ( ex.toString().indexOf("Read time out") != -1) )   { 
			// it's the error we are trying to catch
			// update the counter
			System.err.println(ex.toString()+ " "+counter);
			counter++;
			try { 
			    Thread.sleep(1000);
			} catch(InterruptedException ie) { 
			    
			}
 
			if ( counter > errorThreshold) { 
			    throw ex;
			} 
		    } else { 
			// it's another error
			System.err.println("-not the error we are looking for: "+ex.toString());
			throw ex;
		    }
		} else { 
		    endure = false; // everything's allright
		}
		
	    } // while loop ends
            // set the new value to the cached value.
            mostRecentMessage = rc;
        }
        return rc;
    } // end of getMostRecentMessage().




    /**
     * Convenience function for adding a message to the DFSLog
     * @exception SQLException re-thrown from SQL insert operation.
     */

    public boolean addMessage(String ws, String comp, String label,
            String pty, String msg, boolean markForArchiving )
            throws SQLException {
 
        boolean rc = true;

        SimpleDateFormat formatter
            = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.0");
        String nowTime = formatter.format(new java.util.Date());

        StringBuffer insertString
            = new StringBuffer("INSERT INTO dfslog..recent_message ("
            +" logdt, wsid, compid, labelid, priorid, priorstr,"
            +" logmsg, archive_status)"
           +" VALUES ('"+nowTime+"', ");

        insertString.append(toStringWithEscapes(ws));
        insertString.append(", ");
        insertString.append(toStringWithEscapes(comp));
        insertString.append(", ");
        insertString.append(toStringWithEscapes(label));
        insertString.append(", ");
        insertString.append(""+priorityFromString(pty));
        insertString.append(", ");
        insertString.append(toStringWithEscapes(pty));
        insertString.append(", ");
        insertString.append(toStringWithEscapes(msg));
        insertString.append(", ");
        if (markForArchiving) {
            insertString.append("2");
        } else {
            insertString.append("1");
        }
        insertString.append(")");
        Statement st = null;
        SQLException ex = null;
        try {
            st = getStatement();
            if (st != null) {
                st.executeUpdate(insertString.toString());
            } else {
                rc = false;
            }
        } catch (SQLException e) {
            ex = e;
        } finally {
            if (st != null) {
                releaseStatement(st);
                st = null;
            }
        }
        if (ex != null) {
            throw ex;
        }
        return rc;
    } // end of addMessage().




    private String toStringWithEscapes(String s) {
        StringBuffer sb = new StringBuffer(s);
        int i = 0;
        while (true) {
            char c;
            try {
                c = sb.charAt(i);
                if (c == '\'') {
                    sb.insert(i, '\'');
                    i++;
                }
            } catch (StringIndexOutOfBoundsException e) {
                break;
            }
            i++;
        }
        s = new String("'"+sb.toString()+"'");
        return s;
    }




    public static int priorityFromString(String p) {
        int rc = 0;
        while (rc < priorities.length) {
            if (p.equals(priorities[rc])) {
                break;
            } else {
                rc++;
            }
        }
        return rc;
    }




    /**
     * May return null if there has been no "most recent message".
     */

    public String getRecentMessageWorkstation() {
        return mostRecentMessageWorkstation;
    }


    /**
     * Return the msg_id of the mostRecentMessage.
     */

    public long getRecentMessageId() {
        return mostRecentMessageId;
    }


    /**
     * May return null if there has been no "most recent message".
     */

    public String getRecentMessageComponent() {
        return mostRecentMessageComponent;
    }


    /**
     * May return null if there has been no "most recent message".
     */

    public String getRecentMessageLabel() {
        return mostRecentMessageLabel;
    }


} // end of class DFSLogConnectionServer.

// end of file DFSLogConnectionServer.java
