package org.eso.ohs.core.dbb.client;

// $Id: DbbComboBox.java,v 1.8 2005/05/30 10:00:50 gdonino Exp $

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.event.EventListenerList;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbWidget abstract class,
 * defining a Combo Box widget.
 * The object type should be String, otherwise get and set doesn't work properly
 * 
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * CHANGELOG cmavros 25JUNE02 Add clear option 
 * 
 */

public class DbbComboBox extends DbbWidget implements ItemListener {

	/** Override super class variable because choice is not a JComponent. */
	protected JComboBox widget_;
	
	/** Array of database objects. */
	private Object[] dbArray_;

	private static String CLEAR_SELECTION = DbbSelectionList.CLEAR_SELECTION;

	//----------------------------------------
	// Constructors
	//----------------------------------------

	/** 
	 * Create a DbbComboBox object.
	 * 
	 * @param labeltext	Label text to be displayed.
	 * @param chunk	DbbSqlChunk object
	 *  
	 */
	public DbbComboBox(String labeltext, DbbSqlChunk chunk) {

		super(labeltext, chunk);

		widget_ = new JComboBox();

		// the methods in parent must remain consistent
		super.widget_ = widget_;
		label_.setLabelFor(widget_);
		widget_.addItemListener(this);
		setDefault("");
	}

	/** 
	 * Create a DbbChoice object with a specific item list.
	 * 
	 * @param labeltext		Label text to be displayed.
	 * @param chunk		DbbSqlChunk object
	 * @param items		Items in the display list
	 * @param emptyValue Boolean flag: add the default 
	 *				value to the item list
	 */
	public DbbComboBox(
		String labeltext,
		DbbSqlChunk chunk,
		Object[] items,
		boolean emptyValue) {

		this(labeltext, chunk);

		if (emptyValue) {
			Object[] newItems = addDefault(items);
			widget_.setModel(new DefaultComboBoxModel(newItems));
		} else
			widget_.setModel(new DefaultComboBoxModel(items));
		widget_.setSelectedItem("");
		
	}

	/** 
	 * Create a DbbChoice object with an item list mapped to a list of
	 * specific database values.
	 * 
	 * @param labeltext		Label text to be displayed.
	 * @param chunk		DbbSqlChunk object
	 * @param items		Items in the display list
	 * @param dbArray		Database values mapped to the 
	 *				display item list
	 * @param emptyValue	Boolean flag: add the default 
	 *				value to the item list
	 */
	public DbbComboBox(
		String labeltext,
		DbbSqlChunk chunk,
		Object[] items,
		Object[] dbArray,
		boolean emptyValue) {

		this(labeltext, chunk, items, emptyValue);

		if (emptyValue) {
			dbArray_ = addDefault(dbArray);
		} else {	
			dbArray_ = dbArray;
		}
	}


	//----------------------------------------
	// Private Methods
	//----------------------------------------

	/** 
	 * Add the default value to the item list.
	 *
	 * @param items Input array
	 * @return a modified array after adding the default value.
	 */
	private Object[] addDefault(Object[] items) {
		Vector newValues = new Vector();
		newValues.addElement(CLEAR_SELECTION);
		for (int i = 0; i < items.length; i++) {
			newValues.addElement(items[i]);
		}
		newValues.addElement("");
		Object[] newValArray = new Object[newValues.size()];
		newValues.copyInto(newValArray);
		return newValArray;
	}

	//----------------------------------------
	// Public Methods
	//----------------------------------------
	/**
	 * Set the list of values. They should be string otherwise get and set doen's work properly
	 * @param listData Array of internal items
	 * @param listDisplayData Array of displayed items 
	 * @return the number of data that are displayed
	 */
	public void setListDisplayData(
		Object[] listData,
		Object[] listDisplayData) {

		if (listData != null
			&& listDisplayData != null
			&& listData.length == listDisplayData.length) {
			// reset the widget 
			widget_.removeAllItems();
			dbArray_ = addDefault(listDisplayData);
			listData = addDefault(listData);
			for (int i = 0; i < listData.length; i++) {
				widget_.addItem(listData[i]);
			}

		} else {
			// stdlog_.warn("The parameters are null or have different size");
		}
	}

	/** Add a numeric item to the list.
	 * @param item	Integer item.
	 */
	public void addItem(int item) {
		widget_.addItem(String.valueOf(item));
	}

	/** Add a string item to the list.
	 * @param item	String item.
	 */
	public void addItem(String item) {
		widget_.addItem(item);
	}

	/** Add a numeric item list.
	 * @param item_list	Integer item list.
	 */
	public void addItemList(int[] item_list) {
		for (int i = 0; i < item_list.length; i++) {
			widget_.addItem(String.valueOf(item_list[i]));
		}
	}

	/** Add a string item list.
	 * @param item_list	String item list.
	 */
	public void addItemList(String[] item_list) {
		for (int i = 0; i < item_list.length; i++) {
			widget_.addItem(item_list[i]);
		}
	}
	
	/** clear up.
	 */
	public void removeAll() {
			widget_.removeAllItems();
	}

	
	
	
	
	/** Get the displayed string item.
	 * @return Currently displayed item.
	 */
	public String get() {
		if (dbArray_ != null && dbArray_.length > 0) {
			int selectedIndex = widget_.getSelectedIndex();
			if (selectedIndex == -1) {
				return "";
			}
			if (((String) widget_.getSelectedItem()).equals(CLEAR_SELECTION)) {
				setDefault("");
				widget_.setSelectedItem(String.valueOf(""));
			} else
				return (String) dbArray_[widget_.getSelectedIndex()];
		}
		Object obj = widget_.getSelectedItem();
		String retValue;
		if (obj instanceof Integer) {
			retValue = obj.toString();
		} else {
			retValue = ((String) widget_.getSelectedItem());
		}
		return retValue;
	}

	/** Set the displayed string item. Only if the objects of type
	 * String, otherwise the set doesn't work properly
	 */
	public void set(String value) {
		if (sqlChunk_.getDataType() == Integer.class ) {
				
		}
		widget_.setSelectedItem(value);
	}
	

	/** 
	 * It is necessary to redefine it in order to access the
	 * overridden attribute widget_.
	 *
	 * @return The widget
	 * @see #widget_ 
	 */
	public Component getWidget() {
		return widget_;
	}

	/**
	 * Method to check if the widget contents have been cleared.
	 *
	 * @return A boolean value: <code>true</code> if it is empty,
	 *         <code>false</code> otherwise.  
	 */
	public boolean isEmpty() {
		String sel = get();
		return (sel == null || sel.equals(""));
	}

	/**
	 * Listener interface implementation.
	 * @param ev	 Event triggering the action.
	 */
	public void itemStateChanged(ItemEvent ev) {
		fireDbbWidgetAction();
	}

    /**
     * @param currInstr
     * @return boolean
     */
    public boolean contains(String obj) {
        int numItems = widget_.getItemCount();
        boolean result = false;
        for (int i = 0; i < numItems; i++) {
            Object currObj = widget_.getItemAt(i);
            if (currObj.equals(obj)) {
                result = true;
                break;
            }
        }
          return result;
    }

    /**
     * @param telList
     * @param telList2 void
     */
    public void setListDisplayDataNoListeners(String[] objList, String[] objDisplayList) {
       EventListenerList tmpListenerList = listenerList_;
       listenerList_ = new EventListenerList();
       setListDisplayData(objList,objDisplayList);
       listenerList_ = tmpListenerList;
        
    }
}
