package org.eso.ohs.core.dbb.rmi;

// $Id: DbbLocalQuery.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $

import java.rmi.RemoteException;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;

/**   
 * This is the main engine of a database browser: builds SQL
 * statements, performs queries (within threads), returns result data.
 *
 * @author Bob Kemp
 *
 */
public class DbbLocalQuery implements DbbQuery {

private static final String rcsid = "$Id: DbbLocalQuery.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";

private static Logger stdlog_ = 
    Logger.getLogger( DbbLocalQuery.class );
 
    /** Debug boolean flag */
    public boolean debug = false;

    /** Maximum number of retries when contacting the server */
    private static final int MAX_RETRIES = 10;

    /** Retries counter */
    private int retryCount_ = 0;

    /** Reference to the remote query object */
    private DbbQuery rmiMgr_;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create a DbbLocalQuery object setting the remote query object reference
     * @param dbbQ Remote query object
     */
    public DbbLocalQuery(DbbQuery dbbQ) {
	rmiMgr_ = dbbQ;
    }


    //----------------------------------------
    // Private methods
    //----------------------------------------

    /** 
     * Make sure a connection is ok.
     *
     * @exception RemoteException RMI exception occurs
     */
    private void checkConnection()
	throws RemoteException {
	if (rmiMgr_ == null) {
	    resetServerConnection(null);
	}
    }


    /** 
     * Rest the server connection
     *
     * @param e Remote exception
     *
     * @exception RemoteException RMI exception occurs
     */
    private void resetServerConnection(RemoteException e)
	throws RemoteException {
	if (++retryCount_ > MAX_RETRIES) {
	    retryCount_ = 0;
	    throw e;
	}
	if (e != null) stdlog_.warn("",e);

	try {
	    // First try to get a new StorageManager instance.
	    rmiMgr_ = factory();
	}
	catch (RemoteException exc) {
	    // Lost contact with server?  Try reconnecting.
	    rmiMgr_ = factory();
	}
	stdlog_.debug("Reset server connection: done");
    }


    /**
     * Log a transaction.
     * @param caller Caller
     */
    private void logCompletedTransaction(String caller) {
	if(stdlog_.isDebugEnabled())
	    stdlog_.debug("Completed call to " + caller);
    }


    //----------------------------------------
    // Protected methods
    //----------------------------------------

    /** Factory method. TBD */
    protected DbbRemoteQuery factory()
	throws RemoteException {
	return null;
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Set the number of rows to be displayed per time. 
     * @exception RemoteException RMI exception occurs
     */
    public void setNumOfRowsToGet(int n)
	throws RemoteException {
	for ( ; ; ) {
	    try {
		checkConnection();
		rmiMgr_.setNumOfRowsToGet(n);
		logCompletedTransaction("setNumOfRowsToGet");
		return;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }


    /** 
     * Used internally by the view in order to get the result data
     * structure.
     *
     * @return DbbSqlQueryResult data structure containing the query
     *	       result.
     * @exception RemoteException RMI exception occurs
     */
    public DbbSqlQueryResult getResult()
	throws RemoteException {

	for ( ; ; ) {
	    try {
		checkConnection();
		DbbSqlQueryResult result = rmiMgr_.getResult();

		logCompletedTransaction("getResult");
		return result;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }


    /**
     * Check if the remote query result contains more rows.
     * @return The boolean result of the check.
     * @exception RemoteException RMI exception occurs
     */
    public boolean hasMoreRows()
	throws RemoteException {

	for ( ; ; ) {
	    try {
		checkConnection();
		boolean result = rmiMgr_.hasMoreRows();

		logCompletedTransaction("hasMoreRows");
		return result;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }


    /** 
     * Read rows from the remote query result object.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     */
    public void readRows()
	throws SQLException, RemoteException {

	for ( ; ; ) {
	    try {
		checkConnection();
		rmiMgr_.readRows();

		logCompletedTransaction("hasMoreRows");
		return;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }

    public DbbSqlQueryResult getEmptyResult() throws  RemoteException {

    opNotSupported();
    return null;
    
    }

	private void opNotSupported() throws RemoteException {
		throw new RemoteException("Operation not supported for this object");
		
	}
    
    public DbbSqlQueryResult appendRows(DbbSqlQueryResult queryResult) throws  RemoteException {

   
    opNotSupported();
    return queryResult;
    
    }
    


    /** 
     * Close the remote query result object.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     */
    public void close()
	throws SQLException, RemoteException {
	for ( ; ; ) {
	    try {
		checkConnection();
		rmiMgr_.close();

		logCompletedTransaction("close");
		return;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }
}



