package org.eso.ohs.core.gui.models;

import gnu.regexp.RE;
import gnu.regexp.REException;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import org.apache.log4j.Logger;

/**
 * A Document model for JTextField (and JTextArea) instances that
 * only allows the insertion of text, so that the buffer contents
 * matches a given regular expression.
 * <p>
 * NOTE: We permit Perl-5 style regular expressions.
 * <table>
 * <tr><td> \d </td><td> digit </td></tr>
 * <tr><td> \s </td><td> space or tab </td></tr>
 * <tr><td> x* </td><td> 0 or more occurrences of x </td></tr>
 * <tr><td> x? </td>
 *	<td> either 0 or 1 occurrences of x, ie x{0,1} </td></tr>
 * <tr><td> (xyz) </td><td> grouping of items </td></tr>
 * <tr><td> \\x </td><td> literally x (escape character) </td></tr>
 * <tr><td> x{m,n} </td>
 *	<td>at least m occurrences and at most n occurrences of x</td></tr>
 * </table>
 * 
 * <p>We need a double-backslash (\\) to escape a character
 * because Java uses backslash as an escape character as well.
 * The pattern x{m,n} matches at least m occurrences and at
 * most n occurrences of x.
 * 
 * <p>
 * $Id: InputMaskModel.java,v 1.3 2004/10/13 14:48:08 ddorigo Exp $
 */
public class InputMaskModel extends PlainDocument {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( InputMaskModel.class );

    /** Maximum number of characters allowed in the model, ie buffer size */
    private final static int MAX_MODEL_SIZE = 256;

    /** Regular expression for legal values */
    private String legalValues_;

    /** Compiled regular expression for legal values */
    private RE legalPattern_;

    /** local copy of characters in the model: for efficiency */
    protected StringBuffer buffer_;

    /** workspace for testing whether an insertion produces a legal value */
    private StringBuffer tmpBuffer_;

    /**
     * Constructs an input mask model that will only accept input
     * that leaves the buffer matching the pattern in legalValues.
     * 
     * @param legalValues   regular expression describing legal values
     */
    public InputMaskModel(String legalValues) {
	legalValues_ = legalValues;
	buffer_       = new StringBuffer();
	tmpBuffer_    = new StringBuffer();

// stdlog_.debug("pattern: " + legalValues_);
	try {
	    legalPattern_ = new RE(legalValues_);	    
	}
	catch (REException e) {
	    stdlog_.error(e);
	    throw new IllegalArgumentException(e.getMessage());
	}
    }


    /**
     * Inserts the given string into the text buffer, if and only if
     * it leaves the buffer in a legal state, i.e.\ matching the
     * regexp of legal values.
     * 
     * @param offset	offset in the text buffer at which to insert string.
     * @param str	string to be inserted.
     * @param attr	text attributes.
     * 
     * @exception BadLocationException	an illegal offset was specified.
     */
    public void insertString(int offset, String str, AttributeSet a)
		throws BadLocationException {
        if (str == null) {
            return;
        }
	int slen = str.length();

	    // Construct array containing (possible) new buffer contents,
	    //  then test whether it is in an allowed format.
	tmpBuffer_.setLength(0);
	tmpBuffer_.append(buffer_);
	tmpBuffer_.insert(offset, str);

	if ( !legalPattern_.isMatch(tmpBuffer_)) {
	    return;
	}

	StringBuffer savedBuffer = buffer_;
	buffer_ = tmpBuffer_;
	tmpBuffer_ = savedBuffer;

	super.insertString(offset, str, a);
    }


    /**
     * Removes the specified substring from the text buffer, providing that
     * it will leave the buffer in a legal state.
     * 
     * @param offset	offset in the text buffer at which to insert string.
     * @param len	length of the substring to be removed.
     * 
     * @exception BadLocationException	an illegal offset was specified.
     */
    public void remove(int offs, int len)
		throws BadLocationException {

	tmpBuffer_.setLength(0);
	for (int j=0; j < offs; j++) {
	    tmpBuffer_.append(buffer_.charAt(j));
	}

	for (int j=offs+len, max=buffer_.length(); j < max; j++) {
	    tmpBuffer_.append(buffer_.charAt(j));
	}

	if ( !legalPattern_.isMatch(tmpBuffer_)) {
	    return;
	}

	StringBuffer savedBuffer = buffer_;
	buffer_ = tmpBuffer_;
	tmpBuffer_ = savedBuffer;

	super.remove(offs, len);
    }

}

