package org.eso.ohs.core.utilities.rmi;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.server.Unreferenced;

import org.eso.ohs.core.utilities.Logger;

/**
 * Implements a generic, do-nothing application server. <BR>
 * All application servers should extend this class.
 * 
 * @version $Revision: 1.7 $
 * 
 * @author amchavan, 11-Oct-2001: factored out from AppServer.java and
 *         DbbServer.java
 */
public class OHSServer extends UnicastRemoteObject implements OHSServerControl,
        Unreferenced {

    private static final String rcsid = "$Id: OHSServer.java,v 1.7 2005/05/04 13:26:39 gdonino Exp $";

    /** RMI server port number */
    private int rmiPort_;

    /** RMI server name */
    private String rmiName_;

    /** Full RMI server URL */
    private String rmiURL_;

    /**
     * Logger object for for writing to the log file. <BR>
     * For a smooth shutdown, it should be closed properly at the end.
     */
    private Logger logger_ = null;

    /** Our own private registry (at max one per jvm) */
    protected static Registry registry_ = null;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Assign the parameter to a remote object.For starting up the server.
     * Use the {@link #createAndStart(String,int,Logger) createAndStart} for loading a running Server.
     * <P>
     * 
     * @param name
     *            Name to be used when contacting this server.
     * 
     * @param port
     *            Port number on which the server is going to listen for
     *            connection requests. A value of zero means the system default.
     * 
     * @param logger
     *            Where logging output goes. If this is <code>null</code>,
     *            the server logs to stdout.
     * 
     * @throws AlreadyBoundException
     *             If <em>name</em> or <em>port</em> are not valid.
     * @throws NotBoundException
     *             If <em>name</em> or <em>port</em> are not valid.
     * @throws IOException
     *             If socket factory cannot be created.
     * 
     * @see java.rmi.Naming
     * @see org.eso.ohs.core.utilities.rmi.OHSRMISocketFactory
     * @see org.eso.ohs.core.utilities.Logger
     *  
     */
    public OHSServer(String name, int port, Logger logger) throws RemoteException {
        rmiPort_ = port;
        rmiName_ = name;
        rmiURL_ = computeURL(name, port);
        logger_ = logger == null ? new Logger() : logger;
    }

  
    /**
     * Creates a remote object, which can be addressed by an Uniform Resource
     * Locator (URL). The URL for an object of this class or derivate class is
     * specified using host, port and name: <BR>
     * &nbsp;&nbsp;&nbsp;
     * <code>rmi://</code><em>host</em><code>:</code><em>port</em><code>/</code><em>name</em>
     * <BR>
     * where: <BR>
     * &nbsp;&nbsp;&nbsp; <em>host</em>= name of host where the process is
     * running; <BR>
     * &nbsp;&nbsp;&nbsp; <em>port</em>= port number where the server is
     * listening for connections; <BR>
     * &nbsp;&nbsp;&nbsp; <em>name</em>= name of the server.
     * <P>
     * 
     * @param name
     *            Name to be used when contacting this server.
     * 
     * @param port
     *            Port number on which the server is going to listen for
     *            connection requests. A value of zero means the system default.
     * 
     * @param logger
     *            Where logging output goes. If this is <code>null</code>,
     *            the server logs to stdout.
     * 
     * @return the Server
     * @throws AlreadyBoundException
     *             If <em>name</em> or <em>port</em> are not valid.
     * @throws NotBoundException
     *             If <em>name</em> or <em>port</em> are not valid.
     * 
     * @throws RemoteException
     * @throws MalformedURLException
     * 
     * @see java.rmi.Naming
     * @see org.eso.ohs.core.utilities.rmi.OHSRMISocketFactory
     * @see org.eso.ohs.core.utilities.Logger
     *  
     */ 
   
    public static OHSServer createAndStart(String name, int port, Logger logger)
            throws RemoteException, MalformedURLException,
            AlreadyBoundException, NotBoundException {
        OHSServer server = new OHSServer(name, port, logger);
        server.startUp();
        return server;
    }


    /**
     * Creates a remote object using the parameters provided by the constructor
     * 
     * @throws RemoteException
     * @throws AlreadyBoundException
     * @throws MalformedURLException
     * @throws NotBoundException
     */
    public void startUp() throws RemoteException, AlreadyBoundException,
            MalformedURLException, NotBoundException {

        log("Logging started port: " + rmiPort_);

        // Socked factory creation was moved
        // to main() method of the P2PP AppServer.
        // It is too late here to create the socket
        // factory -- server sockets for this object
        // were already created.
        // @author amchavan, 22-Jan-2002
        //
        // OHSRMISocketFactory sf = new OHSRMISocketFactory();
        // RMISocketFactory.setSocketFactory( sf );
        // log( "Created socket factory" );
        if (registry_ == null) {
            registry_ = getRegistry(rmiPort_);
        }
        log("Created local RMI registry on port " + rmiPort_);

        bind();
        log("Bound as " + getURL());
        log("Server started");
    }

    protected Registry getRegistry(int port) throws RemoteException {
        return LocateRegistry.createRegistry(port);
    }

    /**
     * Bind this server to the RMI registry. <BR>
     * A server cannot operate if it is not bound.
     * 
     * @throws NotBoundException
     * @throws RemoteException
     * @throws AlreadyBoundException
     * @throws MalformedURLException
     */
    public void bind() throws RemoteException, AlreadyBoundException,
            MalformedURLException, NotBoundException {
        Naming.bind(rmiURL_, this);
    }

    /**
     * Bind this server to the RMI registry, overriding any previous binding of
     * the same name: <em>dangerous</em>, to be used only if you know what
     * you are doing. <BR>
     * A server cannot operate if it is not bound.
     * 
     * @throws NotBoundException
     * @throws RemoteException
     * @throws AlreadyBoundException
     * @throws MalformedURLException
     */
    public void rebind() throws RemoteException, AlreadyBoundException,
            MalformedURLException, NotBoundException {
        Naming.rebind(rmiURL_, this);
    }

    /**
     * Unbind this server from the RMI registry. <BR>
     * Servers should always be unbound before terminating to avoid polluting
     * the registry with 'dead' names.
     * 
     * @throws RemoteException
     * @throws AlreadyBoundException
     * @throws MalformedURLException
     * @throws NotBoundException
     */
    public void unbind() throws RemoteException, AlreadyBoundException,
            MalformedURLException, NotBoundException {
        Naming.unbind(rmiURL_);
    }

    /**
     * Field accessor.
     * 
     * @return The <em>name</em> component of the URL for this remote object.
     */
    public String getName() {
        return (rmiName_);
    }

    /**
     * Field accessor.
     * 
     * @return The <em>port</em> component of the URL for this remote object.
     */
    public int getPort() {
        return (rmiPort_);
    }

    /**
     * Field accessor.
     * 
     * @return The full URL for this remote object.
     */
    public String getURL() {
        return (rmiURL_);
    }

    /**
     * Utility method. Computes the full URL of a server running on the current
     * host with the given name and port number
     * 
     * @param name
     *            Name component of the URL.
     * @param port
     *            Port number component of the URL.
     */
    public static String computeURL(String name, int port) {

        String thishost = "localhost";
        String url = null;

        try {
            thishost = InetAddress.getLocalHost().getHostName();
        } catch (java.net.UnknownHostException e) {
            // do nothing -- use the preassigned value for thishost
        }

        url = "rmi://" + thishost + ":" + port + "/" + name;
        return (url);
    }

    /**
     * @return The client's fully qualified host name, if known. <BR>
     *         Otherwise its IP address, if known. <BR>
     *         Otherwise a generic string.
     */
    public String getClientHostName() {

        String host = "<unknown hostname>";
        InetAddress ipadd = null;

        try {
            host = getClientHost(); // something like 134.171.26.185
            ipadd = InetAddress.getByName(host);
            host = ipadd.getHostName();
        } catch (java.net.UnknownHostException e) { /* Do nothing */
        } catch (ServerNotActiveException e) { /* Do nothing */
        }

        return (host);
    }

    /**
     * Field accessor.
     * 
     * @return The internal Logger object. This allows applications direct
     *         access to the Logger API, which is richer than this class'
     *         logging API.
     */
    public Logger getLogger() {
        return (logger_);
    }

    /**
     * Set the Logger object for this server. Any previous instance gets closed.
     * 
     * @param l
     *            The new Logger object. If null, logging is turned off.
     */
    public void setLogger(Logger l) {

        if (logger_ != null)
            logger_.close();
        logger_ = l;
    }

    /**
     * Set the Logger object for this server. Any previous instance gets closed.
     * 
     * @param logfile
     *            Pathname of a writeable file, to which the log will be output.
     * @throws IOException
     */
    public void setLogger(String logfile) throws IOException {

        if (logger_ != null)
            logger_.close();
        logger_ = new Logger(logfile);
    }

    /**
     * Log a message. <BR>
     * This is a shortcut to the Logger API; if no Logger is defined, this
     * method does nothing.
     * 
     * @param msg
     *            The message to log.
     * @see #setLogger
     */
    public void log(String msg) {
        if (logger_ != null)
            logger_.log(msg, false);
    }

    /**
     * Log an exception. <BR>
     * This is a shortcut to the Logger API; if no Logger is defined, this
     * method does nothing.
     * 
     * @param e
     *            The exception to log.
     * @param msg
     *            An extra explanatory message.
     * @see #setLogger
     */
    public void log(String msg, Exception e) {
        if (logger_ != null)
            logger_.logException(msg, e);
    }

    /** @see OHSServerControl#exit( int ) */
    public void exit(int xstat) throws ServerNotActiveException {

        log("Terminated by " + getClientHostName());
        try {
            unbind();
            // log( "Unbound" );
        } catch (Exception e) {
            logger_.logException("Trying to unbind", e);
        }
        log("Exiting (" + xstat + ")");
        System.exit(xstat);
    }

    /** @see OHSServerControl#exit */
    public void exit() throws ServerNotActiveException {
        exit(0);
    }

    /**
     * @see OHSServerControl#ping
     * @throws ServerNotActiveException
     */
    public void ping() throws ServerNotActiveException {
        log("Pinged by " + getClientHostName());
    }

    /**
     * Implements the Unreferenced interface.
     */
    public void unreferenced() {
        log("No clients are connected");
    }

    /**
     * Main program. <BR>
     * Create and run an OHSServer taking the configuration parameters from the
     * array of the arguments.
     * <P>
     * args[0]: server name, for instance <code>TestServ</code><BR>
     * args[1]: server port number, for instance <code>1234</code>
     * <P>
     * 
     * To run this (do-nothing) server:
     * <OL>
     * <LI>make sure your <code>CLASSPATH</code> environment variable is
     * correctly set.
     * <LI><code>cd ...</code> (just above the org/eso/ohs directory tree)
     * <LI><code>java org.eso.ohs.cliserv.OHSServer TestServ 1234</code>
     * </OL>
     */
    public static void main(String[] args) {

        int port;
        String name;
        String msg = "Usage: ...OHSServer server-name server-port";

        if (args.length < 2) {
            System.out.println(msg);
            System.exit(1);
        }

        name = args[0];
        try {
            Integer p = new Integer(args[1]);
            port = p.intValue();

            OHSServer ohss = createAndStart(name, port, null);
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
}

