package org.eso.ohs.instruments;

import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;


/**
 * Instanciates a paramater class of type number(Float). Verifies string input
 * in the GUI to make sure a valid number within a specified range is entered.
 * <p>
 * $Id: NumberParameter.java,v 1.10 2004/04/08 12:09:52 tcanavan Exp $
 * @author Tim Canavan 
 */

public class NumberParameter extends Parameter {

    static final long serialVersionUID = 1652945642248714865L;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( NumberParameter.class );


    /** Min Value */
    
    private Float minValue_;

    /** Max Value */
    private Float maxValue_;

    /** List of valid values */
    private Float[] validValues_;


    /** Minimum value that a value value can be */

    public Float getMinValue() {
        return minValue_;
    }

    /** Maximum value that a value can be */

    public Float getMaxValue() {
        return maxValue_;
    }

    /** A list of valud values*/

    public Float[] getValidValues() {
        return validValues_;
    }

    /** 
     * Tooltip text for the parameter
     * @return Appropiate tooltip text 
     */

   
    public String getToolTipText() {
        String tooltp="Enter valid value";
        boolean range = false;
        if(getMinValue() != null
        && getMaxValue() != null)
            range = true;
        Float[] validValues = getValidValues();
        StringBuffer validFloats = new StringBuffer(20);
        if(validValues != null) {
            for(int i=0;i<validValues.length;i++) {
                validFloats.append(validValues[i].floatValue() + " ");
            }
        }
        StringBuffer rangeBuffer = null;
        if(range) {
             rangeBuffer = new StringBuffer();
             rangeBuffer.append(getMinValue());        
             rangeBuffer.append("..");
             rangeBuffer.append(getMaxValue());
	}
        if(range && validValues == null)
            tooltp = "Floating point within range " + rangeBuffer.toString();
        if(!range && validValues !=null)
            tooltp = "Floating point value from the list " + validFloats.toString();
        if(range && validValues != null)
            tooltp = "Floating point value within " + 
                                        rangeBuffer.toString() + " or in list "
                                        + validFloats.toString();
        if(!range && validValues == null)
            tooltp = "Enter Floating point value";
        return tooltp;               
    }

    /**
     * Constructor for Number parameter
     * @param pMeta Number parameters Meta Data
     */

    public NumberParameter(ParamMetaData pMeta) {
        super(pMeta);
        initValidValues();
    }



    public char getDBTypeCode() {
        return 'N';
    }
    ////////////////////////////////////////////////////////////////
    // Methods                                                     /
    ////////////////////////////////////////////////////////////////

    /**
     * Verify a valid number(Float) has been entered.
     * @param value String value to be verified
     * @return If value is a valid number and in the 
     *         range defined by the paramater class return OK
     *         else NaN invalid if invalid integer value entered
     * 
     */
  
    public boolean verifyValue(String value) {
        Float fvalue = null;
        String range = getRange();
        if(value == null || value.length() <= 0) {
	    return false;
        }
        if(value.compareTo("NODEFAULT")==0) {
             return false;
        }
        try {
            fvalue = new Float(value);
            if(fvalue.isNaN())
                return false;
       }
        catch(NumberFormatException e) {           
            return false;
	}
        if(range!=null && range.length() > 0) {
            if(!valueInRange(fvalue)) {
                return false;
            }
        }
        return true;
    }   


    /**
     * Check to see if a number value is in the specified range
     * A range of 1.5 2..4000 3.5 means that any values between 2 and 400
     * are valid along with 1.5 and 3.5
     * @param value Float value to be verified
     * @return valid Boolean value set to trues if value in range
     */

    private boolean valueInRange(Float value) {
        if(minValue_ != null && maxValue_ != null) {
             if(value.floatValue() >= minValue_.floatValue() && value.floatValue() <= maxValue_.floatValue()) {
                 return true;            
             }
        }
        if(validValues_ != null) {
            for(int i=0;i<validValues_.length;i++) {
                if(value.floatValue() == validValues_[i].floatValue())
                    return true;
            }
        }
        return false;       
    }

    /**
     * Check to see if a number value is in the specified range
     * A range of 1.5 2..4000 3.5 means that any values between 2 and 400
     * are valid along with 1.5 and 3.5
     * @param value Float value to be verified
     * @return valid Boolean value set to trues if value in range
     */

    public void initValidValues() {
        StringTokenizer token = new StringTokenizer(getRange());
        StringBuffer minValueStr = new StringBuffer(20);
        StringBuffer maxValueStr = new StringBuffer(20);
        Vector validValues = new Vector();       
        while(token.hasMoreElements()) {
            String rangeItem  = token.nextToken();
            int index = rangeItem.indexOf("..");
            if(index != -1) {
                int i = index-1;
                while(i >= 0 && rangeItem.charAt(i) != ' ') {
                    minValueStr.insert(0,rangeItem.charAt(i));
                    i--;
                }
                i = index+2;
                while(i < rangeItem.length() && rangeItem.charAt(i) != ' ') {
                    maxValueStr.append(rangeItem.charAt(i));
                    i++;
                }
                try {                   
                    minValue_ = new Float(minValueStr.toString());
                    maxValue_ = new Float(maxValueStr.toString()); 
                 
                }
                catch(NumberFormatException e) {
                    stdlog_.error("Invalid range : " + e.getMessage());                  
                }               
            }
            else {
		try {
                    Float validValue = new Float(rangeItem);
                    validValues.addElement(validValue);
                }
                catch(NumberFormatException e) {
                    stdlog_.error("Invalid range in paramater : " + getParameterName());                  
                }
                    
            }                  
        } 
        if(validValues.size() > 0 ) {
            validValues_ = new Float[validValues.size()];
            validValues.copyInto(validValues_);           
        }
    }

}


