package org.eso.ohs.instruments;

import java.io.IOException;
import java.io.Serializable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This is the class that contains all the data for a given template
 * signature file. It contsins a reference to the read only meta data
 * object. This meta date refernce can be shared accross many Template 
 * Signature objects
 * <p>
 *
 *
 * @version $Revision: 1.88 $
 *
 *
 * @author Tim Canavan 
 */

public class TemplateSignature implements Cloneable, Serializable {

    private static Logger stdlog_ =
        Logger.getLogger(TemplateSignature.class);

    static final long serialVersionUID = -4378250031958131854L;

    private final static String rcsid =
        "$Id: TemplateSignature.java,v 1.88 2005/02/15 09:53:54 pnunes Exp $";
    /**
     * The instrument associated with .tsf file
     *
     */

    private String instrumentName_;

    /**
     * The version of the instrument associates with .tsf file
     */
    private float ipversion_ = -1;

    /**
     *  Template Name for .tsf file
     */

    private String templateName_;

    /**
     * Date and time of the .tsf file
     */

    private long lastModified_;

    /**
     * Mazimum length a parameter value can be used for display
     * purposes
     */

    public static final int PARAM_VALUE_MAXLEN = 2000;


    /**
     * Array of parameter classes associated with a given .tsf
     */

    private Parameter plist[];

    /**
     * Array of parameters to be displayed in the gui. This array holds 
     * references to plist array
     */
    private transient Parameter dispParam[];

    /**
     * Meta Data required to build template signature.
     * This is required for backwards compatibility to Verioion
     * 2.1.x caches. It should not be removed if Version 2.1.x
     * cached still need to be read
     */
    private TemplateSignatureMetaData tsMetaData_;

    /**
     * For efficency reasons the Meta Data class was made transient
     * so it is not saved out every time
     */

    private transient TemplateSignatureMetaData newMetaData_;


    /**
     * Constructor receives its data from the TemplateSignatureMetaData 
     * cash. It then call init which instanciates all the derived 
     * parameter classes and initialises all the member variables of the 
     * TemplateSignature class
     * 
     * @param tsMetaData Template Signature Meta Information
     * @param ipv the ip version of the Template Signature (taken from ISF file).
     */

    public TemplateSignature(TemplateSignatureMetaData tsMetaData, float ipv) {
        newMetaData_ = tsMetaData;
        ipversion_ = ipv;
        init();
    }



    /**
     * Copy constructor for a template signature
     * @param ts Template Signature to be copied
     */

    public TemplateSignature(TemplateSignature ts) {
        this(ts.getMetaData(), ts.getVersion());
        Parameter[] src = ts.getParamList();
        for (int i = 0; i < plist.length; i++) {
            plist[i].setValue(src[i].getValue());
        }
    }

    /**
     * Instanciates all the derived  parameter classes and initialises 
     * all the member variables of the TemplateSignature class     
     */

    private void init() {
        //stdlog_.debug("CLASS TemplateSignature - init:");
        ParamMetaData[] metalist = newMetaData_.getParamList();
        if (metalist != null) {
            initParamList(metalist);
        }
        instrumentName_ = newMetaData_.getInstrumentName();
        templateName_ = newMetaData_.getTemplateName();
        lastModified_ = newMetaData_.getLastModified();
    }

    /**
     * Template has warnings against it 
     * @return true if there is warnings otherwise false
     */

    public boolean getWarningStatus() {
        return newMetaData_.getWarningStatus();
    }

    /**
     * Initialise the Paramerer List from the Meta Data List
     * @param metaList Meta Data for each of the parameters in the list
     */

    private void initParamList(ParamMetaData[] metalist) {
        Parameter p = null;
        Vector newList = new Vector();
        int i = 0;
        stdlog_.debug("CLASS TemplateSignature - initParamList:");
        for (i = 0; i < metalist.length; i++) {
            try {
                p = CreateParam.Create(metalist[i]);
            } catch (NullPointerException e) {
                stdlog_.error(e);
            }
            newList.addElement(p);
        }
        stdlog_.debug("CLASS TemplateSignature - initParamList: loop: i =" + i);
        plist = new Parameter[newList.size()];
        newList.copyInto(plist);
        createDisplayParamList();
    }

    /**
     * Set the meta date for a Template Signature
     * @param metaData Reference to Template Signature Meta Data
     */

    public void setMetaData(TemplateSignatureMetaData metaData) {
        newMetaData_ = metaData;

    }

    /**
     * Recreate the meta data for all the parameters in a .tsf file
     */

    public void recreateParamMetaData() {
        ParamMetaData[] metaList = newMetaData_.getParamList();
        for (int i = 0; i < plist.length; i++) {
            plist[i].setMetaData(metaList[i]);
        }
    }

    /**
     * Return a reference to the meta data associated with a .tsf file
     */

    public TemplateSignatureMetaData getMetaData() {
        return newMetaData_;

    }

    /**
     * Read ts from local storage
     * @param in Stream to capture data from cache
     * @exception IOException IO Exception if record cannot be read from disk
     * @exception ClassNotFoundException class not found exception
     */

    private void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException {
        in.defaultReadObject();

        if (tsMetaData_ != null) {
            // For backwards compatibility with Version 2.1.x cached      
            instrumentName_ = tsMetaData_.getInstrumentName();
            templateName_ = tsMetaData_.getTemplateName();
            lastModified_ = tsMetaData_.getLastModified();
            tsMetaData_ = null;
        }
    }

    /**
      * Write Template Signature to local storage
      * @param out Stream to write data to local storage
      * @exception IOException IO Exception if record cannot be read from disk
      * @exception ClassNotFoundException class not found exception
      */

    private void writeObject(java.io.ObjectOutputStream out)
        throws IOException, ClassNotFoundException {
        out.defaultWriteObject();
    }

    /**
     * List of Parameters assocaited with .tsf file.
     *
     * @return parameter list
     */

    public Parameter[] getParamList() {
        return plist;
    }

    /**
     * List of parameters which can 
     * be displayed on the gui 
     *
     * @return dislplay patametet list
     */

    public Parameter[] getDisplayParamList() {
        return dispParam;
    }

    /**
     * Create the parameter list to be displayed on the gui
     */

    public void createDisplayParamList() {
        Vector displayParamList = new Vector();
        for (int i = 0; i < plist.length; i++) {
            if (plist[i].isDisplay()) {
                displayParamList.addElement(plist[i]);
            }
        }
        dispParam = new Parameter[displayParamList.size()];
        displayParamList.copyInto(dispParam);
    }

    /**
     * Set the parameter list for a Template Signature
     * @param plist List of Parameters.
     */

    public void setParamList(Parameter[] plist) {
        this.plist = new Parameter[plist.length];
        for (int i = 0; i < plist.length; i++) {
            this.plist[i] = plist[i];
        }
        createDisplayParamList();

    }

    /**
     * Return the list of labels associated with each parameter
     * so it can be displayed
     * @return An array of labels that describe each parameter
     */

    public String[] getLabels() {
        String labels[] = new String[dispParam.length];
        for (int i = 0; i < dispParam.length; i++) {
            labels[i] = dispParam[i].getLabel();
        }
        return labels;
    }

    /**
     * Template name for example RILDOT01
     * @return template name
     */

    public String getTemplateName() {
        if (templateName_==null) 
            return("");
        else
            return templateName_;
    }

    /**
     * The last modified date for the Templaate Signature
     * @return The last modified date of the .tsf file
     */

    public long getLastModified() {
        return lastModified_;
    }

    /**
     * Template description
     * @return template description
     */

    public String getTemplateDesc() {
        return newMetaData_.getTemplateDesc();
    }

    /**
     * Template type
     * @return template type
     */

    public String getTemplateType() {

        return newMetaData_.getTemplateType();
    }

    /**
     * Resources
     * @return TPL.RESOURCES ??
     */

    public String getResources() {
        return newMetaData_.getResources();
    }

    /**
     * The Acquisition time only refers to Acquisition templates
     * and it is directly hard coded into the Acquisition template
     * in the TPL.EXECTINE keyword
     * @return  TBD
     */

    public int getAcquisitionTime() {
        if (getTemplateType().equals("acquisition")) {
            return newMetaData_.getExecTime();
        }
        return 0;
    }

    /**
     * Template mode TPL.MODE
     * @return template mode
     */

    public String getMode() {
        return newMetaData_.getMode();
    }


    /**
     * Instrument name
     * name of the instrument for example FORS
     * @return Instrument name
     */

    public String getInstrumentName() {
        if (instrumentName_ == null) 
            return("");
        else
            return instrumentName_;
    }

    /**
     * @return the ip version of the IP in which this TSF is associated,
     * -1 indicates an invalid IP version.
     */
    public float getVersion() {
        return ipversion_;
    }

    /**
     * Set the IP version of this TemplateSignature.
     * Note that this explicitly sets the member in this object, which may well differ
     * from the meta data. It is typically used with the 
     * TemplateSignatureFactory.refreshTemplate() call to convert objects using
     * TemplateSignatures from one IP version to another.
     */
    public void setVersion(float ipv) {
        ipversion_ = ipv;
    }

    /**
     * @return the Instrument object assocaited with this template signature.
     */
    public Instrument getInstrument() {
        return InstrumentList.getInstance().getInstrument(
            getInstrumentName(),
            getVersion());
    }

    /**
     * Check to see if the version number in the cache 
     * is the same as the one on the application server
     * @param ts TemplateSignature 
     * @return If versions are the same return true else
     *         return false.
     */

    public boolean checkVersion(TemplateSignature ts) {
        if (ts.getVersion() == getVersion())
            return true;
        return false;
    }

    /**
     * Set the value of specific parameter in the TemplateSignature 
     * file
     *
     * @param name parameter name
     * @param value value of parameter
     */

    public void setParameterValue(String name, String value) {
        for (int i = 0; i < plist.length; i++) {
            if (plist[i].getParameterName().equals(name)) {
                plist[i].setValue(value);
            }
        }
    }

    /**
     * Verify the list of parameter values entered
     * if any of them are not set to OK add theem to an errorlist
     * containig the name and the error message
     *    
     * @return An array of identifier,data pairs.
     */

    public String[] verify() {
        Vector statusVec = new Vector();
        for (int i = 0; i < dispParam.length; i++) {
            if (dispParam[i].hasHiddenKeyword()) {
                continue;
            }
            if (!dispParam[i].verify()) {
                statusVec.addElement(dispParam[i].getLabel());
            }
        }
        String[] status = new String[statusVec.size()];
        statusVec.copyInto(status);
        return status;
    }

    /**
     * Return a String which will be displayed ot printed out 
     * describing the template and its parametets
     * @return Report String
     */
    public String toReportString() {
        int PAD_PARAM_NAMELEN = 22;
        int PAD_PARAM_VALUE = 20;
        StringBuffer buf = new StringBuffer();
        buf.append(getTemplateName() + ":\n");
        for (int i = 0; i < dispParam.length; i++) {
            String label = dispParam[i].getLabel();
            String value = null;
            value = dispParam[i].getDisplayValue();
            if (label.length() <= PAD_PARAM_NAMELEN)
                buf.append(TextUtils.padString(label, PAD_PARAM_NAMELEN));
            else {
                buf.append(label.substring(0, PAD_PARAM_NAMELEN - 2) + "..");
            }
            buf.append(" ");
            if (value.length() <= PAD_PARAM_VALUE)
                buf.append(TextUtils.padString(value, PAD_PARAM_VALUE));
            else {
                buf.append(value.substring(0, PAD_PARAM_VALUE - 2) + "..");
            }

            if (i > 0 && i % 2 != 0) {
                buf.append("\n");
            } else
                buf.append(" ");

        }
        buf.append("\n");
        return buf.toString();
    }



    /**
     * Get the value of a given parameter, or empty string if not found
     * @param key the parameter name to locate
     * if it is not found in the param set, the meta data is searched.
     * @return string containing the value for the parameter, of "" if not
     * found.
     */

    public String getParameterValue(String key) {
        return getParameterValue(key, true);
    }





    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < plist.length; i++) {
            sb.append(plist[i].toString());
        }
        return sb.toString();
    }

    /**
     * Get the value of a given parameter, or empty string if not found
     * @param key the parameter name to locate
     * @param includeMetaData if true, search meta data for the keyword
     * if it is not found in the param set - default is true.
     * @return string containing the value for the parameter, of "" if not
     * found.
     */

    public String getParameterValue(String key, boolean searchMetaData) {
        String rc = null;
        Parameter[] params = getParamList();
        for (int i = 0; i < params.length; i++) {
            if (params[i].getParameterName().equals(key)) {
                rc = params[i].getValue();
                break;
            }
        }
        if (rc == null && newMetaData_ != null && searchMetaData) {
            // search the meta data
            PafData pdl[] = newMetaData_.getIdentifierList();
            for (int i = 0; pdl != null && i < pdl.length; i++) {
                if (pdl[i].getIdentifier().equals(key)) {
                    rc = pdl[i].getValue();
                }
            }
        }
        if (rc == null) {
            rc = "";
        }
        return rc;
    }
    
    
    /**
     * Compares two <code>TemplateSignature</code> objects,
     * testing <code>parameterName_</code> and <code>value_</code> fields of the <code>Parameter</code> objects 
     * in <code>dispParam[]</code> field. 
     * 
     * @param other
     * @return boolean
     */
    public boolean equals( TemplateSignature other ) {
        Parameter [] plistThis ;
        Parameter [] plistOther ;
        Parameter p;
        String name,value;
        
        if ( templateName_.compareTo(other.getTemplateName()) != 0 )
            return (false);
        
        else {
            plistThis = getDisplayParamList();
            plistOther = other.getDisplayParamList();
            
            for (int i = 0; i < plistThis.length; i++) {
                p = plistThis[i];                
                
                name = p.getParameterName();
                value = other.getParameterValue(name);
                
                if (  value.compareTo("") == 0 )
                    return(false);
               
                if ( p.getValue().compareTo(value) != 0 )
                    return(false);                                               
            }
        }            
        
        return(true);
                        
    }
}
