package org.eso.ohs.persistence;

import java.util.Enumeration;
import java.util.Stack;
import java.util.Vector;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.dfs.StorableObject;

/**
 * This subclass of the ObjectManager handles all application-specific
 * issues relating to storage of objects.
 * 
 * <p>
 * For correct operation, there should be at most one instance of this
 * object.  Thus it is a singleton and can only be instantiated using
 * the static Persistence.getInstance() method.  Each time this
 * method is called it will return the <em>same</em> instance of the
 * ObjectManager.
 * 
 * <p>
 * %Id: Persistence.java,v 1.16 2001/06/22 11:47:57 tschlich Exp %
 * <br>
 * @author Bob Kemp
 */
public abstract class OHSPersistence extends ObjectManager {

    private final String rcsid =
        "$Id: OHSPersistence.java,v 1.15 2005/01/14 13:57:00 tcanavan Exp $";

    /** current user id */
    protected int userId_ = -1;

    /**
     * Sets the id of the current user.
     * 
     * @param userId	the id of the end-user
     * 
     * @exception ObjectIOException
     *			if there was an I/O error while setting the userId
     */
    public void setUserId(int userId) throws ObjectIOException {
        Enumeration enum = mediaMgr_.keys();
        Media device;

        userId_ = userId;

        while (enum.hasMoreElements()) {
            device = (Media) enum.nextElement();

            ((StorageManager) mediaMgr_.get(device)).setUserId(userId);
        }
    }

    /**
     * Get the user id in the ObjectManager.
     */
    public int getUserId() {
        return userId_;
    }

    /**
     * Returns a copy of the given BusinessObject after adding it
     * to the Registry.
     * 
     * @param device	the storage medium to be managed
     * @param dir	directory to contain the copy object
     * @param bo	BusinessObject to copy
     * 
     * @exception ObjectIOException
     * 			thrown if there is an I/O error while registering
     * 
     * @see BusinessObject#deepCopy
     */
    public BusinessObject copy(
        Media device,
        DirectoryNode dir,
        BusinessObject bo)
        throws ObjectIOException {
        return copy(device, dir, bo, 0);
    }

    /**
     * Returns a copy of the given BusinessObject after adding it
     * to the Registry.
     * 
     * @param device	the storage medium to be managed
     * @param dir	directory to contain the copy object
     * @param bo	BusinessObject to copy
     * @param baseId	Id on which to base the new object id,
     * 			(StorageManager-dependent)
     * 
     * @exception ObjectIOException
     * 			thrown if there is an I/O error while registering
     * 
     * @see BusinessObject#deepCopy
     */
    public BusinessObject copy(
        Media device,
        DirectoryNode dir,
        BusinessObject bo,
        long baseId)
        throws ObjectIOException {
        BusinessObject boCopy = bo.copy();

        register(device, dir, boCopy, baseId);

        return boCopy;
    }

    /**
     * Deletes each element of the given tree of business objects
     * from the Registry and external storage.
     * 
     * @param bo	top-level BusinessObject of 'tree' to delete
     * 
     * @exception ObjectIOException
     * 			thrown if there is an I/O error while registering
     * @exception ObjectNotFoundException
     *			if an object was not found.
     */
    public void recursiveDelete(BusinessObject topLevelBO)
        throws ObjectIOException, ObjectNotFoundException {
        BusinessObject[] descendants = allDescendantsOf(topLevelBO);

        for (int j = 0; j < descendants.length; j++) {
            BusinessObject busObj = descendants[j];
            RegistryEntry oi = registry_.lookup(busObj.getId());

            deleteBusObj(oi.getMedia(), busObj.getId(), busObj.getClass());
        }
    }

    protected BusinessObject[] allDescendantsOf(BusinessObject topLevelBO) {
        Vector descendants = new Vector();
        Stack stk = new Stack();

        stk.push(topLevelBO);
        while (!stk.isEmpty()) {
            BusinessObject busObj = (BusinessObject) stk.pop();
            BusinessObject[] children = busObj.getBusObjChildren();

            descendants.addElement(busObj);

            for (int j = 0; j < children.length; j++) {
                stk.push(children[j]);
            }
        }

        BusinessObject[] result = new BusinessObject[descendants.size()];
        descendants.copyInto(result);

        return result;
    }

    /**
     * Utility method to set the DbaseId properties of a tree of local
     * BOs to the ids of the corresponding BOs in the database.
     * 
     * @param local	root of tree of BOs on local storage.
     * @param dbase	root of tree of BOs on dbase.
     */
    protected void setDbaseIds(BusinessObject local, BusinessObject dbase) {
        // 	System.out.println("Setting DbaseId for: " + local
        // 			    + " to " + local.getDbaseId());
        local.setDbaseId(dbase.getId());

        BusinessObject[] dbChildren = dbase.getBusObjChildren();
        BusinessObject[] locChildren = dbase.getBusObjChildren();

        for (int j = 0; j < dbChildren.length; j++) {
            // System.out.print("CLASS p2pp.Persistence - setDbaseIds: ");
            // System.out.println(locChildren[j].getId()+" to "+dbChildren[j].getId());
            setDbaseIds(locChildren[j], dbChildren[j]);
        }

    }

    /**
     * Returns the Medium on which the object is stored.  Throws an
     * IllegalArgumentException if it is not currently in the Registry.
     *
     * @param id	id of a StorableObject
     * 
     * @return medium on which it is stored.
     * 
     * @exception IllegalArgumentException
     * 			if the object is not known to the system,
     * 			ie not in Registry.
     */
    public Media getMediaFor(long id) throws IllegalArgumentException {
        RegistryEntry oi = registry_.lookup(id);

        if (oi == null) {
            throw new IllegalArgumentException("Object not in Registry: " + id);
        }
        return oi.getMedia();
    }

    /**
     * Returns the Medium on which the object is stored.  Throws an
     * IllegalArgumentException if it is not currently in the Registry.
     *
     * @param sobj	StorableObject whose storage medium is required
     * 
     * @return		Storage medium containing the object
     * 
     * @exception IllegalArgumentException
     * 			if the object is not known to the system,
     * 			ie not in Registry.
     */
    public Media getMediaFor(StorableObject sobj)
        throws IllegalArgumentException {
        RegistryEntry oi = registry_.lookup(sobj.getId());

        if (oi == null) {
            throw new IllegalArgumentException(
                "Object not in Registry: " + sobj);
        }
        return oi.getMedia();
    }

    /**
     * Returns the named ObservingRun.  Throws an IllegalArgumentException
     * if it is not currently in the Registry.
     *
     * @param device	Medium for which the user should be authenticated
     * @param passwd	Password for the user
     * 
     * @return		true if the user authenticated okay, false otherwise
     * 
     * @exception ObjectIOException
     *			if there was an I/O error while setting the userId
     */
    public boolean authenticateUser(Media device, String passwd)
        throws ObjectIOException {
        StorageManager stMgr = getStorageManager(device);

        return stMgr.loginUser(passwd);
    }

    /**
     * Returns the directory in which the object is stored.  Throws an
     * IllegalArgumentException if it is not currently in the Registry.
     *
     * @param sobj  StorableObject whose containing directory is required
     * 
     * @return      Directory containing the object
     * 
     * @exception IllegalArgumentException
     *          if the object is not known to the system,
     *          ie not in Registry.
     */
    public String getRootFolderPath(ObservingRun obsRun)
        throws IllegalArgumentException, ObjectNotFoundException {
        RegistryEntry oi = registry_.lookup(obsRun.getId());
        if (oi == null) {
            throw new IllegalArgumentException(
                "Object not in Registry: " + obsRun);
        }
        LocalDirMgr dirMgr = (LocalDirMgr) getStorageManager(Media.LOCAL);
        String path = dirMgr.locateRunPath(obsRun.getId(), obsRun.getClass());
        return path;
    }
    
    /**
     * Populate the finding chart objects
     * @param device
     * @param boId
     * @return Array of finding charts
     * @throws ObjectIOException
     */

    public FindingChart[] populateFindingCharts(Media device, long boId)
        throws ObjectIOException {
        return getStorageManager(device).populateFindingCharts(boId);
    }
    
    /**
     * Populate thr readme file objects.
     * @param device
     * @param obsRun
     * @return Readme file
     * @throws ObjectIOException
     */
    
    public Readme populateReadmeFile(Media device,ObservingRun obsRun)
    throws ObjectIOException {
        return getStorageManager(device).populateReadmeFile(obsRun);
    }
    
    /**
     * Populate thr readme file that is on Paranal.
     * @param device
     * @param obsRun
     * @return Readme file.
     * @throws ObjectIOException
     */
    
    public Readme populateReadmeFileReplicate(Media device,ObservingRun obsRun)
    throws ObjectIOException {
        return getStorageManager(device).populateReadmeFileReplicate(obsRun);
    }

    
    /**
     * Populate thr readme file objects.
     * @param device
     * @param obsRun
     * @throws ObjectIOException
     */
    
    public void setReadmeFileStatus(Media device,String status,long runId)
    throws ObjectIOException {
        getStorageManager(device).setReadmeFileStatus(status,runId);
    }
    
    


}
