package org.eso.ohs.persistence;

import java.util.EventObject;


/**
 * Encapsulates information describing changes to a table of objects,
 * and is used to notify listeners of the change.
 *
 * <p>
 * $Id: ObjectTableEvent.java,v 1.1 2005/02/11 09:51:57 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class ObjectTableEvent extends EventObject {

    /**
     * Constant to identify an OBJECT_ADDED event     
     */
    public static final int OBJECT_ADDED    = 1;

    /**
     * Constant to identify an OBJECT_CHANGED event     
     */
    public static final int OBJECT_CHANGED   = 3;

    /**
     * Constant to identify an OBJECT_REMOVED event     
     */

    public static final int OBJECT_REMOVED = 5;

   /**
    * Contains a value of one of the constants defined above
    */
    private int type_;

    /**
     * Id of the object that has changed
     */
    private long id_;

    /**
     * Type of the object that has changed
     */
    private Class objType_;
   
    /**
     * The destination media where the objects will be saved to
     * be it local storage,RDBMS etc..
     */
    private Media device_;

    /**
     * This is the directory node that specifies where the
     * StorableObject is located in local storage.
     */
    private DirectoryNode dir_;


    /**
     * Constructor for ObjectTableEvent.
     * 
     * @param type  	type of event: OBJECT_CHANGED, OBJECT_CREATED, etc
     * @param device	medium on which the object resides.
     * @param dir	directory in which the object resides.
     * @param id	id of the object that has changed.
     * @param objType	type of the object that has changed.
     */
    public ObjectTableEvent(int type, Media device, DirectoryNode dir,
					long id, Class objType) {
	super(new Long(id));
	type_       = type;
	id_	    = id;
	objType_    = objType;
	device_     = device;
	dir_        = dir;
    }


    /**
     * Retrieves ObjectTableEvent type. 
     * @return Type of event OBJECT_CREATED,OBJECT_DELETED
     */
    public int getType() {
	return type_;
    }

    /**
     * Retrieves media type 
     * @return Device where Storable object will be stored.
     */
    public Media getMedia() {
	return device_;
    }

   /**
    * Retrieves the DirectoryNode for the Storable Object
    * @return Location in Local Storage where Storable Object will
    *              be stored.  
    */
    public DirectoryNode getDirectoryNode() {
	return dir_;
    }

    /**
     * Retrieves the id of the StorableObject linked with the ObjectTableEvent.
     *
     * @return id of the object affected
     */
    public long getObjectId() {
	return id_;
    }

    /**
     * Retrieves the class of the StorableObject linked with the event.
     *
     * @return class of the object affected
     */
    public Class getObjectClass() {
	return objType_;
    }

}

