package org.eso.ohs.persistence;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Hashtable;

import org.eso.ohs.core.utilities.EmptyEnumeration;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.instruments.TemplateVersionInfo;

/**
 * The abstract parent class of all storage managers.
 * Defines operations that the ObjectManager can expect of
 * storage managers for all media.
 * 
 * <p>
 * $Id: StorageManager.java,v 1.38 2004/12/14 17:47:35 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 * 
 * @see ObjectManager
 * @see LocalDirMgr
 */

public interface StorageManager {

	/**
	 * Get the root node of the DirectoryNode tree representing
	 * the directories found on this medium.  For example, a
	 * representation of the root of the local StorableObject
	 * storage subtree.
	 * 
	 * @return a representation of the root node
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error while determining the
	 * 			directory hierarchy.
	 */
	public DirectoryNode getRoot() throws ObjectIOException;

	/**
	 * Verify that the TemplateSignature files used by the StorageManager
	 * are the same version as those specified (and used by the client)
	 * 
	 * @param tsList	array of version info objects for each template
	 *			used in an OB
	 * 
	 * @return a list of the names of TemplateSignatures that are out of date.
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error while checking
	 */
	public String[] verifyTemplateSignatures(TemplateVersionInfo[] tsList)
		throws ObjectIOException;

	/**
	 * Verify that the TemplateSignature files used by the StorageManager
	 * are the same version as those specified (and used by the client)
	 * 
	 * @param tsList	array of version info objects for each template
	 *			used in an OB
	 * 
	 * @return a list of the names of TemplateSignatures that are out of date.
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error while checking
	 */
	public FindingChart [] populateFindingCharts(long boId)
		throws ObjectIOException;



    
   
    
    

	/**
	 * @return the requested IP as a byte array, or null if the IP
	 * does not exist on the server.
	 * @exception ObjectIOException when there is an I/O exception.
	 */
	public byte[] downloadInstrumentPackage(String instrument, float version)
		throws ObjectIOException;

	/**
	 * Get the type of media maintained by this StorageManager instance.
	 * 
	 * @return an instance of the Media class.
	 */
	public Media getMedia();

	/**
	 * Calculates a unique id for a StorableObject.
	 *
	 * @return Unique id
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error during the operation
	 */
	public long getNewId(Class cls, long baseId) throws ObjectIOException;

	/**
	 * Initialise the data structures for the appropiate media. For 
	 * local storage this will include reading the directory structure.
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error during the operation
	 */
	public void initialise() throws ObjectIOException;

	/**
	 * Returns a DirectoryNode representing the path within the
	 * storage medium of an object with the given id and type.
	 * For media that do not support directories, the root directory
	 * may be returned.
	 * 
	 * @param id	the id of the StorableObject.
	 * @param cls	the type of stored StorableObject.
	 * 
	 * @return Directory in which to find the object.
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error during the operation
	 * 
	 * @exception ObjectNotFoundException
	 *			if the object was not found.
	 */
	public DirectoryNode find(long id, Class cls)
		throws ObjectNotFoundException, ObjectIOException;

	/**
	 * Sets the id of the current user.
	 * 
	 * @param userId	the id of the end-user
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error while setting the userId
	 */
	public void setUserId(int userId) throws ObjectIOException;

	/**
	 * Gets the id of the current user.
	 * 
	 * @return	the id of the end-user
	 */
	public int getUserId();

	/**
	 * Authenticates the end-user.
	 * 
	 * @param password	password for the current user
	 * 
	 * @return	true if user was successfully authenticated, false otherwise
	 * 
	 * @exception ObjectIOException
	 *			if there was an I/O error while authenticating
	 */
	public boolean loginUser(String password) throws ObjectIOException;

	/**
	 * Returns an array of objects representing the contents of an
	 * object with the given id and type from the chosen directory.
	 * If the medium does not support directories, the <code>node</code>
	 * parameter is ignored.
	 * 
	 * <p>
	 * The array of objects contains:
	 * <ul>
	 *     <li> as the first (0th) element, an instance of a
	 *          StorableObject subclass; followed by
	 *     <li> an alternating sequence of values:
	 *          <ul>
	 *              <li> at even indexes, a property name, ie a string X
	 *                   such that setX() and getX() methods exist.
	 *              <li> at odd indexes, either an instance of
	 *                   java.lang.Long containing the the id of the
	 *                   StorableObject subclass to which the property
	 *                   should refer or a reference to the subclass itself.
	 *          </ul>
	 * </ul>
	 * 
	 * @param node	directory from which the object should be loaded.
	 * @param id	id of the StorableObject.
	 * @param cls	type of StorableObject.
	 * 
	 * @return the array of objects and object references,
	 * 			as described above.
	 * 
	 * @exception ObjectNotFoundException
	 *			if the object was not found.
	 * @exception ObjectIOException
	 * 			if an I/O error occurred while reading the object
	 * 
	 * @see #write
	 * @see #update
	 */
	public UnpackedStorableObject read(DirectoryNode node, long id, Class cls)
		throws ObjectIOException, ObjectNotFoundException;

	/**
	 * Writes an array of objects with the given id and type to
	 * this storage medium, in the chosen directory.  If the medium
	 * does not support directories, the <code>node</code> parameter
	 * is ignored.
	 * 
	 * @param node	the directory in which the object should be saved.
	 * @param id	the id under which to store the StorableObject.
	 * @param cls	the type of StorableObject.
	 * @param obj	the array of objects and object references to store,
	 *			as described in the section for the read method.
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred while writing the object
	 * 
	 * @see #read
	 * @see #update
	 * @see #multiStore
	 */
	public void write(
		DirectoryNode node,
		long id,
		Class cls,
		UnpackedStorableObject obj)
		throws ObjectIOException;

	/**
	 * Updates an array of objects with the given id and type on
	 * this storage medium, in the chosen directory.  If the medium
	 * does not support directories, the <code>node</code> parameter
	 * is ignored.
	 * 
	 * @param node	the directory in which the object should be saved.
	 * @param id	the id under which to store the StorableObject.
	 * @param cls	the type of StorableObject.
	 * @param obj	the array of objects and object references to store,
	 *			as described in the section for the read method.
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred while updating the object
	 * 
	 * @see #read
	 * @see #write
	 * @see #multiStore
	 */
	public void update(
		DirectoryNode node,
		long id,
		Class cls,
		UnpackedStorableObject obj)
		throws ObjectIOException;

	/**
	 * Deletes an object with the given directory, id and type from
	 * this storage medium.  If the medium does not support
	 * directories, the <code>node</code> parameter is ignored.
	 * 
	 * @param node	directory from which the object should be deleted.
	 * @param id	id of the StorableObject.
	 * @param cls	type of StorableObject.
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 * 
	 * @see #read
	 * @see #update
	 * @see #multiStore
	 */
	public void delete(DirectoryNode node, long id, Class cls)
		throws ObjectIOException;

	/**
	 * Moves an object between directories on this storage medium.
	 * 
	 * @param id	id of the StorableObject.
	 * @param cls	type of StorableObject.
	 * @param fromNode	directory from which the object should be moved.
	 * @param toNode	directory to which the object should be moved.
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred while during the operation
	 * 
	 * @exception ObjectNotFoundException
	 *			if the object was not found in fromNode.
	 */
	public void move(
		long id,
		Class cls,
		DirectoryNode fromNode,
		DirectoryNode toNode)
		throws ObjectIOException, ObjectNotFoundException;

	/**
	 * Create a directory on the external medium.
	 *
	 * @param node The directory to be created
	 * @param name The name of the directory to be created.
	 * 
	 * @return True on success false on error
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 */
	public boolean createDirectory(DirectoryNode node, long id)
		throws ObjectIOException;

	/**
	 * Move a directory.
	 *
	 * @param node	directory to be moved
	 * @param toNode	the new parent directory
	 * 
	 * @return True on success false on error
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 * 
	 * @exception ObjectNotFoundException
	 *			if the object was not found in fromNode.
	 */
	public boolean moveDirectory(DirectoryNode node, DirectoryNode toNode)
		throws ObjectNotFoundException, ObjectIOException;

	/**
	 * Delete a directory from external media
	 *      
	 * @param node	Directory to be deleted
	 * 
	 * @return True on success, false on failure
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 */
	public boolean deleteDirectory(DirectoryNode node)
		throws ObjectIOException;

	/**
	 * Returns a list of Summary objects associated with a directory.
	 * 
	 * @param node	Directory to be used to build up list of objects
	 * 
	 * @return Array of Summary[] objects describing the directory contents
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 */
	public Summary[] listObjects(DirectoryNode node) throws ObjectIOException;

	/**
	 * Check in the specified objects in a single atomic operation.
	 * 
	 * @param	usos	Descriptions of each StorableObject to be checked in.
	 * 
	 * @return		array of error strings.
	 *			If length = 0, no errors.
	 */
	public void checkIn(MultiObjSpec[] moi) throws ObjectIOException;
    
    
    /**
     * Check in the readme file into the repository. 
     * 
     * @param  Readmefile object to be checked in.
     * 
     */
    public void checkInReadme(ObservingRun obsRun) throws ObjectIOException;
    
    
    /**
     * Populate the complete readme file including 
     * .usg comments ob comments and the stadnard 
     * readme attributes. 
     * @param  Readmefile object to be checked in.
     * 
     */
    public Readme populateReadmeFile(ObservingRun obsRun) throws ObjectIOException;
    
    
    /**
     * Populate the complete readme file including 
     * that will be displayed on Paranal 
     * readme attributes. 
     * @param  Readmefile object to be checked in.
     * 
     */
    public Readme populateReadmeFileReplicate(ObservingRun obsRun) throws ObjectIOException;

    
    
    /**
     * Set the status for a readme file.  
     */
    public void setReadmeFileStatus(String status,long runId) throws ObjectIOException;


    
    
    /**
     * Check in the readme file into the repository. 
     * 
     * @param  Readmefile object to be checked in.
     * 
     */
    public void usdSubmit(ObservingRun obsRun) throws ObjectIOException;

    
    

	/**
	 * Do a EVM verification test for the specified objects in a single atomic operation.
	 *
	 * @param   usos    Descriptions of each StorableObject to do EVM verification
	 *
	 * @return boolean if OB passed EVM test
	 */
	public boolean verifyObject(MultiObjSpec[] moi) throws ObjectIOException;

	/**
	 * Check out the ObsBlk or CalBlk with the specified id plus
	 * any attached objects, ie Target, ObsDescr, etc.
	 * 
	 * @param	id	id of the ObsBlk or CalBlk to be checked out
	 * 
	 * @return		CheckOutStatus object that contains a return
	 *			status and (if successful) references to the
	 *			checked-out objects.
	 */
	public void checkOut(IdType[] idTypes)
		throws ObjectNotFoundException, ObjectIOException;
    
    
    /**
     * Checkout an individual readme from the database. 
     * @param id of the run which is equivalen to the readme id. 
     */
    public String checkOutReadme(ObservingRun obsRun)
        throws ObjectIOException;
    
    

	/**
	 * Write several objects to the external media in an atomic operation.
	 *      
	 * @param moi	Array of object descriptions.
	 * 
	 * @exception ObjectIOException
	 * 			if an I/O error occurred during the operation
	 * 
	 * @see #read
	 * @see #write
	 * @see #update
	 */
	public void multiStore(MultiObjSpec[] moi) throws ObjectIOException;

	/**
	 * Utility class for passing arguments to multiStore() method.
	 */
	public static class MultiObjSpec implements Serializable {

		/** Version id for object serialization */
		static final long serialVersionUID = -3402500148904197774L;

		/** Has the object already been written, ie write() or update()? */
		private boolean isNew;

		/** Directory in which to store the object */
		public DirectoryNode dir;

		/** Type of the object */
		private Class cls;

		/**
		 * Disassembled form of the object
		 * 
		 * @see ObjectManager#reassembleReferences
		 * @see ObjectManager#disassembleReferences
		 */
		public UnpackedStorableObject obj;

		public StorableObject getStorableObject(long id) {
			//StorableObject obj=null;  
			if (obj.getObject().getId() == id)
				return obj.getObject();
			return null;
		}

	   /**
		* This creates a calibration block / obsblock from a multiObjectList
		* @param multiObjSpec An array of business objects including a calibration block or
		*                     obsBblock and its associated business objects
        * The ObservingRun data will be attached if phase1StorageMgr is not null.
        * 
        * This method used to be in ObjectManager, however it is used by the AppServer application,
        * which has no ObjectManager set up....and so the method has been added here.
		*
		*/

		public static CalibrationBlock fromMultiObjSpecToCalBlock(
                MultiObjSpec[] multiObjSpec, StorageManager phase1StorageMgr)
			throws IllegalArgumentException, IOException {
			CalibrationBlock cb = null;
			boolean found = false;
			long obsRunId = -1;
			for (int i = 0; i < multiObjSpec.length; i++) {
				StorableObject rootObject = multiObjSpec[i].obj.getObject();
				if (rootObject instanceof CalibrationBlock) {
					found = true;
					Enumeration enum = multiObjSpec[i].obj.getProperties();
					while (enum.hasMoreElements()) {
						String propName = (String) enum.nextElement();
						long id = multiObjSpec[i].obj.getNestedObjId(propName);
						if (propName.equals("ObsRun")) {
							obsRunId = id;
						}
						for (int j = 0; j < multiObjSpec.length; j++) {
							StorableObject sobj =
								multiObjSpec[j].getStorableObject(id);
							if (sobj != null) {
								try {
									Class cl = rootObject.getClass();
									Method getMeth =
										cl
											.getMethod(
												"get" + propName,
												new Class[] {
									});
									Method setMeth =
										cl.getMethod(
											"set" + propName,
											new Class[] {
												 getMeth.getReturnType()});
									setMeth.invoke(
										rootObject,
										new Object[] { sobj });
								} catch (IllegalAccessException e) {
									//stdlog_.error(e);
									throw new IOException(e.getMessage());
                                } catch (InvocationTargetException e) {
									//stdlog_.error(e.getTargetException());
									throw new IOException(e.getMessage());
								} catch (NoSuchMethodException e) {
									//stdlog_.error(e);
									throw new IllegalArgumentException(
										"corrupt data: " + propName);
								}
								break;
							}
						}
					}
					cb = (CalibrationBlock) rootObject;
				}
			}

			if (obsRunId != -1 && phase1StorageMgr != null) {
				try {
                    UnpackedStorableObject objs = phase1StorageMgr.read(null, obsRunId, ObservingRun.class);
                    ObservingRun obsRun = (ObservingRun) objs.getObject();
					cb.setObsRun(obsRun);
				} catch (ObjectIOException e) {
					throw new IOException(e.getMessage());
				} catch (ObjectNotFoundException e) {
					throw new IOException(e.getMessage());
				}
			}

			if (!found) {
				throw new IllegalArgumentException("CalBlock/ObsBlock not found in multi obj list");
			}
			cb.refreshTemplates(false);
			return cb;
		}

        /**
         * Returns the isNew.
         * @return boolean
         */
        public boolean isNew() {
            return isNew;
        }

        /**
         * Sets the isNew.
         * @param isNew The isNew to set
         */
        public void setNew(boolean isNew) {
            System.out.println("Is New" + isNew + " Class " + cls);
            //if(isNew == false && cls.equals("FindingCharts.class"))
            this.isNew = isNew;
        }

        /**
         * Returns the cls.
         * @return Class
         */
        public Class getCls() {
            return cls;
        }

        /**
         * Sets the cls.
         * @param cls The cls to set
         */
        public void setCls(Class cls) {
            this.cls = cls;
        }

	}

	public static class UnpackedStorableObject implements Serializable {

		/** Version id for object serialization */
		static final long serialVersionUID = 4696495710324895272L;

		private StorableObject sobj_;
		private Hashtable nestedObjects_;

		public UnpackedStorableObject(StorableObject sobj) {
			if (sobj == null) {
				throw new IllegalArgumentException("UnpackedStorableObject: StorableObject was null");
			}
			sobj_ = sobj;
		}

		public StorableObject getObject() {
			return sobj_;
		}

		public void addNestedObjId(String propertyName, StorableObject sobj) {
			addNestedObjId(propertyName, sobj.getId());
		}

		public void addNestedObjId(String propertyName, long id) {
			if (nestedObjects_ == null) {
				nestedObjects_ = new Hashtable();
			}
			nestedObjects_.put(propertyName, new Long(id));
		}

		public boolean isKnown(String propertyName) {
			return (
				nestedObjects_ != null
					&& nestedObjects_.get(propertyName) != null);
		}

		public long getNestedObjId(String propertyName) {
			Long objId;

			if (nestedObjects_ != null) {
				objId = (Long) nestedObjects_.get(propertyName);
				if (objId != null) {
					return objId.longValue();
				}
			}

			return 0;
		}

		public Enumeration getProperties() {
			return nestedObjects_ == null
				? new EmptyEnumeration()
				: nestedObjects_.keys();
		}

		private boolean checkPropertyExists(String propertyName) {
			Class cl = sobj_.getClass();
			try {
				Method getMeth =
					cl.getMethod("get" + propertyName, new Class[] {
				});
				Method setMeth =
					cl.getMethod(
						"set" + propertyName,
						new Class[] { getMeth.getReturnType()});
				return true;
			} catch (NoSuchMethodException e) {
				return false;
			}
		}

	}

	public static class IdType implements Serializable {

		/** Version id for object serialization */
		static final long serialVersionUID = -9161241158498319561L;

		public long id;
		public Class objType;

		public IdType(long id, Class objType) {
			this.id = id;
			this.objType = objType;
		}

		public int hashCode() {
			return (objType == null)
				? (int) id
				: (int) (id ^ objType.hashCode());
		}

		public boolean equals(Object obj) {
			if (obj instanceof IdType) {
				IdType idt = (IdType) obj;
				return idt.objType == objType && idt.id == id;
			}
			return false;
		}

		public String toString() {
			return "IdType[id=" + id + ",objType=" + objType + "]";
		}

	}


}
