package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.EphemerisFile;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
/**
 * I/O class for OBs stored in the database.
 *
 * <p>
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 *
 * Added execution_time to the database read, write, and update methods
 * <br>ChangeLog: cmavros change made 06/Sept/2001
 */

public class DbaseHandlerCB extends DbaseIOHandler {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(DbaseHandlerCB.class);

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    /**
     * Calibration type
     * @return Calinration type
     */
    protected String getType() {
        return CalibrationBlock.CALBLK_TYPE;
    }

    /**
     * Managed class for Calibration block
     * @return Managed class
     */
    public Class getManagedClass() {
        return CalibrationBlock.class;
    }

    /**
     * Reads additional Information that is not in Caliubration
     * block
     *
     * @param rs  Result set from read of obs_blocks table
     * @param cb  OB to be initialised with additional ObservationBlock info
     * @param vec Object Vector that initialises the extra objects that an OB
     *            refers to
     * @exception SQLException JDBC Exception
     */
    protected void readAdditional(
        ResultSet rs,
        CalibrationBlock ob,
        UnpackedStorableObject uso)
        throws SQLException {
    }

    /**
     * Gets Calibration requirements
     * @param cb reference to CalibrationBlock 
     * @return calibration requirements 
     */
    protected String getCalibrationRequirements(CalibrationBlock cb) {
        return "";
    }

    /**
     * Return the dbase id to reference the attached TimeIntervals.
     * 
     * @param	cb  CalibrationBlock (or sublclass with TIs);
     */
    protected long getTimeIntId(CalibrationBlock cb) {
        return 0;
    }
    
    /**
     * Return the dbase id to reference the attached STTimeIntervals.
     * 
     * @param   cb  CalibrationBlock (or sublclass with TIs);
     */
    protected long getSTTimeIntId(CalibrationBlock cb) {
        return 0;
    }


    /**
     * IP version
     * @return version of the instrument package referred to by the OB
     * <br>ChangeLog: amchavan, 22-Oct-2001: now returns proper IP version.
     */
    protected float getIPVersion(CalibrationBlock cb) {
        return cb.fetchIPVersionFromOd();
    }

    /**
     * Gets ObsGrade value
     * @param cb reference to CalibrationBlock 
     * @return obsGrade value
     */
    protected String getObsGrade(CalibrationBlock cb) {
        return "";
    }

    /**
    * Reads the object from the database, partially reassembling it.
    * References to other business objects are stored in other fields
    * of the array
    * 
    * @param stmt  JDBC Statement for contacting the dbase
    * @param id    per-table id of the object to be retrieved.
    * @return	    array in the format required by reassembleReferences()
    * 
    * @exception SQLException
    *		    an error occurred during the dbase operation
    * @exception ObjectNotFoundException
    *		    the requested object was not found in the database
    * @exception ObjectIOException
    *		    Object I/O Exception
    * 
    * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
    * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
    */
    public UnpackedStorableObject read(Statement stmt, long id)
        throws ObjectNotFoundException, IOException, ObjectIOException, SQLException {
        ResultSet rs = null;
        final CalibrationBlock bo = (CalibrationBlock) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(bo);
        float ipversion = -1;
        stdlog_.debug("!!!!!!!! ID = " + id);
        String sql =
            "SELECT "
                + obrepDbName_
                + "..obs_blocks.* , "
                + obrepDbName_
                + "..obs_descriptions.instrument, "
                + obrepDbName_
                + "..obs_descriptions.ipversion"
                + " FROM "
                + obrepDbName_
                + "..obs_blocks, "
                + obrepDbName_
                + "..obs_descriptions"
                + " WHERE "
                + obrepDbName_
                + "..obs_blocks.ob_id ="
                + id
                + " AND ob_id = "
                + obrepDbName_
                + "..obs_descriptions.od_id"
                + " AND type = '"
                + getType().toUpperCase()
                + "'";

        /* commented this out since is a suspected performance killer
         *
        * + "       OR type = '" + getType().toLowerCase() + "')";
         */
        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sql =
                sql
                    + " AND "
                    + obrepDbName_
                    + "..obs_blocks.user_id ="
                    + userId_;
        }
        stdlog_.debug(
            "CLASS DbaseHandlerCB.UnpackedStorableObject - read: sql = " + sql);

        rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one OB with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        bo.setId(
            Config.getCfg().tableToUniqueId(
                rs.getLong("ob_id"),
                getManagedClass()));
        bo.setDbaseId(rs.getLong("ob_id"));
        readAdditional(rs, bo, uso);

        uso.addNestedObjId("Od",
        // OD now has same id as OB
        Config.getCfg().tableToUniqueId(
            rs.getLong("ob_id"),
            ObservationDescription.class));

        uso.addNestedObjId(
            "ObsRun",
            Config.getCfg().tableToUniqueId(
                rs.getInt("run_id"),
                ObservingRun.class));

        bo.setCreationDate(
            new Long(rs.getString("creation_date").trim()).longValue());
        bo.setName(rs.getString("item_name"));
        bo.setObsComments(rs.getString("obsComments").trim());
        if(rs.getString("inst_comments")!=null)
            bo.setInstrumentComments(rs.getString("inst_comments").trim());
        bo.setStandard(rs.getBoolean("repeats"));
        bo.setStatus(rs.getString("status"));
        bo.setUserPriority(rs.getInt("userPriority"));
        bo.setExecutionTime(rs.getInt("execution_time"));
        bo.setUserComments(rs.getString("userComments").trim());
        // Note that at this time, there is no OD, and therefore the IP
        // of the OB is not correctly set. This does not occur until the
        // OD is read from the database.
        bo.putIPVersionToOd(rs.getFloat("ipversion"));
        ipversion = rs.getFloat("ipversion");

        bo.setCloneParentId(
            new Long(rs.getString("local_storage_id").trim()).longValue());
        bo.setOwnerId(rs.getInt("user_id"));

        // ModTime stored as a string but will be auto converted.
        long lastModTime = rs.getLong("last_mod_date");
        String instrument = rs.getString("instrument");
        long tiId = rs.getInt("ti_id");

        rs.close();

        if (ObservationBlock.class.equals(getManagedClass())) {
            ObservationBlock obsblk = (ObservationBlock) bo;
            Instrument inst =
                InstrumentList.getInstance().getInstrument(
                    instrument,
                    ipversion);
            // handle the case that the instrument is not in the InstrumentList:
            if (inst == null) {
                if (ipversion < 0) {
                    ipversion = 0;
                }
                throw new ObjectNotFoundException(
                    "The OB could not be loaded as the Instrument Package "
                        + instrument
                        + " v"
                        + ipversion
                        + " was not found.");
            }
            TemplateSignature[] tsList = TemplateSigDBIO.read(stmt, inst, id);

            if (tsList.length >= 1 && tsList[0] != null) {
                obsblk.setTemplateSignature(tsList[0]);
                if (inst.hasMasks())
                    MaskDBIO.read(stmt, tsList[0], 0, id);
            } else {
                /**cmavros 19-10-01 
                  Error message is handled in dfs/ObservationBlock.java
                throw new ObjectIOException( "No Acquisition template for " + id);
                */
            }

            TimeInterval[] tiList = TimeIntervalDBIO.read(stmt, tiId,TimeInterval.STANDARD_TI_TYPE);
            obsblk.setTimeIntervals(tiList);
            tiList = TimeIntervalDBIO.read(stmt, tiId,TimeInterval.SIDEREAL_TI_TYPE);
            obsblk.setSTTimeIntervals(tiList);            
        }

        // Set last_modification_date last because altering
        //   other fields sets it to the current date/time.
        bo.setLastModifiedDate(lastModTime);
        stdlog_.debug("@@@@@@@@@ Database id " + bo.getDbaseId());
        return uso;
    }

    /**
    * Reads the object from the database, partially reassembling it.
    * References to other business objects are stored in other fields
    * of the array
    * 
    * @param stmt  JDBC Statement for contacting the dbase
    * @param id    per-table id of the object to be retrieved.
    * @return       array in the format required by reassembleReferences()
    * 
    * @exception SQLException
    *           an error occurred during the dbase operation
    * @exception ObjectNotFoundException
    *           the requested object was not found in the database
    * @exception ObjectIOException
    *           Object I/O Exception
    * 
    * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
    * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
    */
    public UnpackedStorableObject read(Connection con, long id)
        throws ObjectNotFoundException, IOException, ObjectIOException, SQLException {
        Statement stmt = con.createStatement();
        UnpackedStorableObject obj = read(stmt, id);
        CalibrationBlock cb = (CalibrationBlock) obj.getObject();
        // Maybe for operations mode
        if (cb instanceof ObservationBlock) {
            ObservationBlock ob = ((ObservationBlock) cb);
            FindingChart[] fcs = FindingChartsDBIO.read(con, id,false);
            ob.setFindingCharts(fcs);
            EphemerisFile eph = EphFilesDBIO.read(con,id);
            if(eph.hasEphemerisFile())
                 ob.setEphemerisFile(eph);
        }
        return obj;
    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param uso   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject uso)
        throws SQLException, ObjectIOException {
        CalibrationBlock cb = (CalibrationBlock) uso.getObject();
        Config cfg = Config.getCfg();
        long runId = cfg.uniqueToTableId(uso.getNestedObjId("ObsRun"));
        if (cb instanceof ObservationBlock) {
            ObservationBlock ob = ((ObservationBlock) cb);
            TimeInterval[] tiVls = ob.getTimeIntervals();
            if (getTimeIntId(cb) != 0) {
                TimeIntervalDBIO.write(stmt, id, tiVls,TimeInterval.STANDARD_TI_TYPE);
            }            
			tiVls = ob.getSTTimeIntervals();
			if (getTimeIntId(cb) != 0) {
				TimeIntervalDBIO.write(stmt, id, tiVls,TimeInterval.SIDEREAL_TI_TYPE);
			}
            ObservationDescription od = ob.getOd();
            stdlog_.debug("!!!!!!!!!! OD Name " + od.getName());
        }
        ObservingRun or = cb.getObsRun();
        if(AppConfig.getAppConfig().canCheckInVMOBs(or.getInstCode())
        && or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_VISITOR))
            cb.setStatus(CalibrationBlock.ACCEPTED);
        int execTime = cb.getExecutionTime();
        StringBuffer sb = new StringBuffer();

        sb.append(id);
        sb.append(",");
        sb.append(cb.getOwnerId());
        sb.append(",");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("ObsRun")));
        sb.append(",");
        sb.append(cb.getStandard() ? 1 : 0);
        sb.append(",");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("Target")));
        sb.append(",");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("ConstraintSet")));
        sb.append(",");
        sb.append(getTimeIntId(cb));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cb.getName()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cb.getObsComments()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cb.getUserComments()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cb.getInstrumentComments()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(getCalibrationRequirements(cb)));
        sb.append(",");
        sb.append(cb.getUserPriority());
        sb.append(",");
        if (execTime == 0)
            sb.append("NULL");
        else
            sb.append(cb.getExecutionTime());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cb.getStatus()));
        sb.append(",");
        sb.append("'");
        sb.append(getType());
        sb.append("'");
        sb.append(",");
        sb.append(cb.getCreationDate());
        sb.append(",");
        sb.append(cb.getLastModifiedDate());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(getObsGrade(cb)));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString("NOT USED"));
        sb.append(",");
        sb.append("'");
        sb.append(cb.getCloneParentId());
        sb.append("'");

            String sql =
                "INSERT INTO obs_blocks\n"
                    + " (ob_id, user_id, run_id, repeats, tp_id, cs_id, ti_id,"
                    + " item_name, obsComments, userComments,inst_comments , calibrationReq,"
                    + " userPriority, execution_time, status, type,"
                    + " creation_date, last_mod_date, obs_grade,"
                    + " instrPkgVersion," // this column is not required, but a value must be
        // set at the column defn does not allow NULL values
        // and we are relucant to change the table defn.
    +" local_storage_id)\n" + " VALUES (" + sb + ")";

        stdlog_.debug("dbase.phase2.DbaseHandlerCB.write: " + sql);
        stmt.executeUpdate(sql);

        if (cb instanceof ObservationBlock) {
            ObservationBlock ob = ((ObservationBlock) cb);
            TemplateSignature at = ob.getTemplateSignature();

            if (at != null) {
                TemplateSigDBIO.write(
                    stmt,
                    id,
                    new TemplateSignature[] { at },
                    true);
                Instrument instrument = at.getInstrument();
                if (instrument.hasMasks())
                    MaskDBIO.write(stmt, at, 0, id);
            }
            ProcessExtraConstraints.write(stmt,ob.getConstraintSet(),ob.getOd().getInstrument(),ob.getOd().getIPVersion(),id);
        }

    }

    public void write(Connection con, long id, UnpackedStorableObject uso)
        throws SQLException, ObjectIOException {
        Statement stmt = con.createStatement();
        write(stmt, id, uso);
        CalibrationBlock cb = (CalibrationBlock) uso.getObject();
        if (cb instanceof ObservationBlock) {
            long runId =
                Config.getCfg().uniqueToTableId(uso.getNestedObjId("ObsRun"));
            ObservationBlock ob = ((ObservationBlock) cb);
            FindingChart[] fc = ob.getFindingCharts();
            if (fc == null) {
                stdlog_.debug("NO FINDING CHARTS AVAILABLE");
            }
            FindingChartsDBIO.write(con, id, runId, fc);
            if(ob.hasEphemerisFile())
                 EphFilesDBIO.write(con,id,ob.getEphemerisFile());
        }
    }
    
    public void update(Connection con, long id, UnpackedStorableObject uso)
        throws SQLException, ObjectIOException {
        Statement stmt = con.createStatement();
        update(stmt, id, uso);
        CalibrationBlock cb = (CalibrationBlock) uso.getObject();
        if (cb instanceof ObservationBlock) {
            long runId =
                Config.getCfg().uniqueToTableId(uso.getNestedObjId("ObsRun"));
            ObservationBlock ob = ((ObservationBlock) cb);
            FindingChart[] fc = ob.getFindingCharts();
            if (fc == null) {
                stdlog_.debug("NO FINDING CHARTS AVAILABLE");
            }
            FindingChartsDBIO.update(con, id, runId, fc);
        }
    }

    
    

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param uso   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject uso)
        throws SQLException {
        CalibrationBlock cb = (CalibrationBlock) uso.getObject();
        Config cfg = Config.getCfg();

        if (cb instanceof ObservationBlock) {
            ObservationBlock ob = ((ObservationBlock) cb);
            TimeInterval[] tiVls = ob.getTimeIntervals();

            if (getTimeIntId(cb) != 0) {
                TimeIntervalDBIO.update(stmt, id, tiVls,TimeInterval.STANDARD_TI_TYPE);
            }            
            tiVls = ob.getSTTimeIntervals();
            if (getTimeIntId(cb) != 0) {
                TimeIntervalDBIO.update(stmt, id, tiVls,TimeInterval.SIDEREAL_TI_TYPE);
            }

        }

        StringBuffer sb = new StringBuffer();

        sb.append("ob_id = ");
        sb.append(id);
        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sb.append(", user_id = ");
            sb.append(userId_);
        }
        sb.append(", run_id = ");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("ObsRun")));
        sb.append(", repeats = ");
        sb.append(cb.getStandard() ? 1 : 0);
        sb.append(", tp_id = ");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("Target")));
        sb.append(", cs_id = ");
        sb.append(cfg.uniqueToTableId(uso.getNestedObjId("ConstraintSet")));
        sb.append(", ti_id = ");
        sb.append(getTimeIntId(cb));

        sb.append(", item_name = ");
        sb.append(SqlUtils.sqlEscapedString(cb.getName()));
        sb.append(", obsComments = ");
        sb.append(SqlUtils.sqlEscapedString(cb.getObsComments()));
        sb.append(", userComments = ");
        sb.append(SqlUtils.sqlEscapedString(cb.getUserComments()));
        sb.append(", inst_comments = ");
        sb.append(SqlUtils.sqlEscapedString(cb.getInstrumentComments()));
        sb.append(", calibrationReq = ");
        sb.append(SqlUtils.sqlEscapedString(getCalibrationRequirements(cb)));
        sb.append(", userPriority = ");
        sb.append(cb.getUserPriority());
        sb.append(", execution_time = ");
        sb.append(cb.getExecutionTime());
        sb.append(", status = ");
        sb.append(SqlUtils.sqlEscapedString(cb.getStatus()));
        sb.append(", type = ");
        sb.append("'");
        sb.append(getType());
        sb.append("'");

        sb.append(", creation_date = ");
        sb.append(cb.getCreationDate());
        sb.append(", last_mod_date = ");
        sb.append(cb.getLastModifiedDate());
        sb.append(", obs_grade = ");
        sb.append(SqlUtils.sqlEscapedString(getObsGrade(cb)));
        sb.append(", local_storage_id = ");
        sb.append("'");
        sb.append(cb.getCloneParentId());
        sb.append("'");
        String sql;
        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sql =
                "UPDATE obs_blocks"
                    + " SET "
                    + sb
                    + " WHERE ob_id = "
                    + id
                    + " AND user_id = "
                    + userId_;
        } else {
            sql = "UPDATE obs_blocks" + " SET " + sb + " WHERE ob_id = " + id;
        }

        stdlog_.debug("dbase.phase2.DbaseHandlerCB.update: " + sql);
        stmt.executeUpdate(sql);

        if (cb instanceof ObservationBlock) {
            ObservationBlock ob = ((ObservationBlock) cb);
            TemplateSignature at = ob.getTemplateSignature();

            if (at != null) {
                TemplateSigDBIO.update(
                    stmt,
                    id,
                    new TemplateSignature[] { at },
                    true);
            }            
        }
    }
    
    
    
    public static String getObTypeFromDatabaseId(long obId) throws SQLException {
        Connection con = null;
        Statement stmt = null;
        Config cfg = Config.getCfg();
        SQLException error = null;
        String type=null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();
            String sql =
                "SELECT "
                    + obrepDbName_
                    + "..obs_blocks.type"
                    + " FROM "
                    + obrepDbName_
                    + "..obs_blocks "
                    + "where "
                    + obrepDbName_
                    + "..obs_blocks.ob_id = "
                    + obId;
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next()) {
                type = rs.getString("type");
            }
        } catch (SQLException e) {
            error = e;
        }        
        finally {
            ConnectionPool.getPool().releaseConnection(con);
        }
        if(error != null)
             throw new SQLException(error.getMessage());
        if (type == null) {
            throw new IllegalArgumentException("No Template Exists on the database for this OB");
        }
        return type;
    }
    

    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id)
        throws ObjectIOException, SQLException {

        String sqlText =
            "DELETE FROM obs_blocks"
                + " WHERE ob_id = "
                + id
                + " AND user_id = "
                + userId_; 

        stdlog_.debug(sqlText);
        stmt.executeUpdate(sqlText);

        TemplateSigDBIO.delete(stmt, id, true);
        MaskDBIO.delete(stmt, id);
        TimeIntervalDBIO.delete(stmt, id,TimeInterval.STANDARD_TI_TYPE);
        TimeIntervalDBIO.delete(stmt, id,TimeInterval.SIDEREAL_TI_TYPE);
        FindingChartsDBIO.delete(stmt,id);
        EphFilesDBIO.delete(stmt,id);
    }

}
