package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for CSs stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerCS.java,v 1.2 2005/11/08 15:53:36 ddorigo Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerCS extends DbaseIOHandler {


    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(DbaseHandlerCS.class);

    private static String obrepDbName =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerCS() {
    }

    public Class getManagedClass() {
        return ConstraintSet.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
        throws ObjectNotFoundException, IOException, SQLException {
        ResultSet rs = null;
        ConstraintSet cs = (ConstraintSet) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(cs);
        ProcessExtraConstraints.read(stmt,id,cs);

        String sql =
            "SELECT * FROM  "
                + obrepDbName
                + "..constraint_sets"
                + " WHERE cs_id ="
                + id;

        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sql =
                sql
                    + " AND "
                    + obrepDbName
                    + "..constraint_sets.user_id = "
                    + userId_;
        }
        stdlog_.debug("CLASS DbaseHandlerCS - read: sql = " + sql);

        rs = stmt.executeQuery(sql);

        if (!rs.next()) {
            throw new ObjectNotFoundException("Object not found in database");
        }
        stdlog_.debug(
            "CLASS DbaseHandlerCS - read: Name = " + rs.getString("item_name"));
        cs.setId(
            Config.getCfg().tableToUniqueId(
                rs.getLong("cs_id"),
                ConstraintSet.class));
        cs.setName(rs.getString("item_name"));
        cs.setCloneParentId(rs.getLong("local_storage_id"));
        cs.setOwnerId(rs.getInt("user_id"));
        cs.setDescription(rs.getString("item_description").trim());
        // As these are compulsory we should leave them be
        // This is for backwards compatability

        cs.setSeeing(rs.getFloat("seeing"));
        cs.setSkyTransparency(
            ConstraintSet.fromDbSkyTransparency(
                rs.getString("sky_transparency")));
        cs.setAirmass(rs.getFloat("airmass"));
        cs.setFractionalLunarIllumination(rs.getFloat("fli"));
        cs.setMoonAngularDistance(rs.getInt("moon_distance") / 3600000);
        cs.setStrehlRatio(rs.getFloat("strehl_ratio"));

        rs.close();

        return uso;
    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
        throws SQLException {
        ConstraintSet cs = (ConstraintSet) obj.getObject();
        stdlog_.debug("CLASS DbaseHandlerCS - write:");
        String instrument = null;
        float ipv = 0;
        StringBuffer buf = new StringBuffer();
        buf.append("SELECT instrument,ipversion from obs_descriptions where od_id = " + id);
        ResultSet rs = stmt.executeQuery(buf.toString());
        while (rs.next()) {
            ipv = rs.getFloat("ipversion");
            instrument = rs.getString("instrument");
            if (ipv < 0) {
                // a NULL value in the database is set to 0.0.
                ipv = 0;
            }
        }


        StringBuffer sb = new StringBuffer();
        sb.append(id);
        sb.append(",");
        sb.append(cs.getOwnerId());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cs.getName()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(cs.getDescription()));
        sb.append(",");
        sb.append(cs.getSeeing());
        sb.append(",");
        sb.append(
            SqlUtils.sqlEscapedString(
                SkyTransparency.toDbSkyTransparency(cs.getSkyTransparency())));
        sb.append(",");
        sb.append(cs.getAirmass());
        sb.append(",");
        sb.append(cs.getFractionalLunarIllumination());
        sb.append(",");
        sb.append(cs.getMoonAngularDistance() * 3600000);
        sb.append(",");
        sb.append("'");
        sb.append(cs.getCloneParentId());
        sb.append("'");
        sb.append(",");
        sb.append(cs.getStrehlRatio());

        String sqlText =
            "INSERT INTO constraint_sets "
                + " (cs_id, user_id, item_name, item_description,"
                + "  seeing, sky_transparency, airmass, fli,"
                + "  moon_distance, local_storage_id, strehl_ratio"
                + " )"
                + " VALUES ("
                + sb
                + ")";

        stdlog_.debug("CLASS DbaseHandlerCS - write: sqlText = " + sqlText);
        stmt.executeUpdate(sqlText);
    }

    /* Writes the object to the database, for the first time.
        * 
        * @param stmt  JDBC Statement for contacting the dbase
        * @param id    per-table id of the object to be written.
        * @param obj   an array: the first element is the BO to be stored.
        * @exception SQLException  an error occurred during the dbase operation
        * 
        * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
        * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
        */
    public void write(Connection conn, long id, UnpackedStorableObject obj)
        throws SQLException {
        Statement stmt = conn.createStatement();
        write(stmt, id, obj);
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
        throws SQLException {
        ConstraintSet cs = (ConstraintSet) obj.getObject();

        StringBuffer sb = new StringBuffer();

        sb.append("cs_id = ");
        sb.append(id);
        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sb.append(", user_id = ");
            sb.append(userId_);
        }
        sb.append(", item_name = ");
        sb.append(SqlUtils.sqlEscapedString(cs.getName()));
        sb.append(", item_description = ");
        sb.append(SqlUtils.sqlEscapedString(cs.getDescription()));

        sb.append(", seeing = ");
        sb.append(cs.getSeeing());
        sb.append(", sky_transparency = ");
        sb.append(
            SqlUtils.sqlEscapedString(
                SkyTransparency.toDbSkyTransparency(cs.getSkyTransparency())));
        sb.append(", airmass = ");
        sb.append(cs.getAirmass());
        sb.append(", fli = ");
        sb.append(cs.getFractionalLunarIllumination());
        sb.append(", moon_distance = ");
        sb.append(cs.getMoonAngularDistance() * 3600000);
        sb.append(", local_storage_id = ");
        sb.append("'");
        sb.append(cs.getCloneParentId());
        sb.append("'");
        sb.append(", strehl_ratio = ");
        sb.append(cs.getStrehlRatio());
        String sql = null;
        if (!Config.getCfg().isOperationsModeId(userId_)) {
            sql =
                "UPDATE constraint_sets"
                    + " SET "
                    + sb
                    + " WHERE cs_id = "
                    + id
                    + " AND user_id = "
                    + userId_;
        } else {
            sql =
                "UPDATE constraint_sets"
                    + " SET "
                    + sb
                    + " WHERE cs_id = "
                    + id;
        }

        stdlog_.debug("CLASS DbaseHandlerCS - update: sql = " + sql);
        stmt.executeUpdate(sql);
        ProcessExtraConstraints.update(stmt,id,cs);
    }

    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id) throws SQLException {

        String sqlText =
            "DELETE FROM constraint_sets"
                + " WHERE cs_id = "
                + id
                + " AND user_id = "
                + userId_;

        stdlog_.debug("CLASS DbaseHandlerCS - delete: sqlText = " + sqlText);
        stmt.executeUpdate(sqlText);

    }

}
