package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.DbaseStorageMgr;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;

/**
 * The storage manager class for files stored in the database.
 * A concrete subclass of StorageManager.
 * <p>
 * NOTE: we could not instantiate the Sybase driver in the normal way, using
 * Class.forName().  Instead it is necessary to follow the special instruction
 * in the Sybase JDBC driver installation instructions.  This is to update the
 * Java system properties.
 * <p>
 * $Id: DbaseMgr.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * @version $Revision: 1.1 $
 * <br>
 * ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer
 */

public class DbaseMgr extends DbaseStorageMgr {

    private static Logger stdlog_ = Logger.getLogger(DbaseMgr.class);

    /**
     * Check and reject OBs on checkIn that are not built with the
     * IP version specified in the Obs Run for that OB.
     */
    private boolean enforceObsRunIPVersionOnCheckIn = true;

    /**
     * Array of period numbers. OBs may only be checked if their
     * Observing Run is in a period in this set.
     * If the array is empty or null, all periods are permitted.
     */
    private int[] acceptablePeriods = null;

    /**
     * The order in which objects must be inserted into the database
     * due to integrity constraints.  For deletion, (needless to say?)
     * the order should be reversed.
     */
    private static final Class[] insertionOrder_ =
        {
            Target.class,
            ConstraintSet.class,
            ObservationBlock.class,
            CalibrationBlock.class,
            ObservationDescription.class,
            ObservingRun.class};

    /**
     * Default constructor
     */
    public DbaseMgr() {
        super(Media.PHASE2_DB);
    }

    /**
     * Set the enforeObsRunIPVersionOnCheckIn flag
     */
    public void setEnforceIPVersionCheck(boolean b) {
        enforceObsRunIPVersionOnCheckIn = b;
    }

    /**
     * @return the enforeObsRunIPVersionOnCheckIn flag
     */
    public boolean getEnforceIPVersionCheck() {
        return enforceObsRunIPVersionOnCheckIn;
    }

    protected void createRegistry() {
        registry_ = new Phase2HandlerRegistry();
    }

    /**
     * Returns the cached JDBC database Connection object creating
     * it first, if necessary.
     *
     * @return  JDBC Connection object
     *
     * @exception SQLException
                        an error occurred during the dbase operation
     * @exception ObjectIOException
     *                  an error occured during an access to a business object
     */
    protected Connection acquireConnection() throws ObjectIOException, SQLException {
        return acquireConnection(CalibrationBlock.class);
    }

    /**
     * Internal procedure to produce summaries of objects of a given type.
     * The results are appended to the list <code>sumVec</code>
     * passed as a parameter.
     *
     * @param stmt      JDBC Statement to use for contacting the dbase
     * @param sumVec    vector containing the summaries
     * @param objType   type of object to search for
     * @param table     name of the database table to search
     *
     * @exception SQLException an error occurred during the dbase operation
     */
    void addObjectFileEntry(
        Statement stmt,
        Vector sumVec,
        Class objType,
        String table,
        String nonOpsMode,
        String extraClauses)
        throws SQLException {
        	
        Config cfg = Config.getCfg();
        String dbTables = cfg.summaryDbaseTables(objType);
        String[] dbColumns = cfg.summaryDbaseColumns(objType);
        String[] colProps = cfg.folderColumnProperties(objType);
        StringBuffer buf = new StringBuffer();
        Summary sum;

        for (int j = 0; j < dbColumns.length; j++) {
            buf.append(dbColumns[j]);
            if (j < dbColumns.length - 1) {
                buf.append(",");
            }
        }

        // Note: DISTINCT is _important_ for handling ObsRuns !!
        //       Otherwise, P2PP will run out of memory!
        String sql = "SELECT DISTINCT " + buf + " FROM " + dbTables;

        if (cfg.isOperationsModeId(getUserId())) {
            if (extraClauses.length() > 0) {
                sql = sql + " WHERE " + extraClauses;
            }
        } else {
            sql = sql + " WHERE " + nonOpsMode;

            if (extraClauses.length() > 0) {
                sql = sql + " AND " + extraClauses;
            }
        }

        stdlog_.debug("sql = " + sql);

        ResultSet rs = stmt.executeQuery(sql);

        while (rs.next()) {
            sum = new Summary();
            // sum.setId(Config.getCfg().tableToUniqueId(rs.getInt("id"), objType));
            // sum.setName(rs.getString("item_name"));
            sum.setObjectClass(objType);
            for (int j = 0; j < dbColumns.length; j++) {
                Object obj = rs.getObject(j + 1);

                // XXX This is an Appalling Hack!!
                // XXX Colnames shouldn't be hardcoded but I need the type!
                if (dbColumns[j].equals("id")
                    || dbColumns[j].endsWith("_id")
                    || dbColumns[j].endsWith(".id")) {
                    sum.setId(
                        Config.getCfg().tableToUniqueId(
                            ((Number)obj).longValue(),
                            objType));

                    sum.setPropertyValue(colProps[j], new Long(sum.getId()));
                } else {
                    sum.setPropertyValue(colProps[j], obj);
                }
            }

            sumVec.addElement(sum);
        }
        rs.close();
    }

    /**
     * @return the requested IP as a byte array, or an empty (size 0) array
     * if the IP does not exist on the server.
     * @exception ObjectIOException when there is an I/O exception.
     */
    public byte[] downloadInstrumentPackage(String instrument, float version)
        throws ObjectIOException {
        try {
            return InstrumentPackageDBIO.getIPData(instrument, version);
        } catch (SQLException esql) {
            throw new ObjectIOException(esql.getMessage());
        } catch (IOException eio) {
            throw new ObjectIOException(eio.getMessage());
        }
    }

    /**
     * Do a EVM verification test for the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to do EVM verification
     *
     * @return boolean if OB passed EVM test
     */

    public boolean verifyObject(MultiObjSpec[] mos) throws ObjectIOException {
        return true;
    }

    /**
     * Check in the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to be checked in.
     *
     * @return  array of error strings.
     *          If length = 0, no errors.
     */
    public void checkIn(MultiObjSpec[] mos) throws ObjectIOException {
        MultiObjSpec[] orderedMOS = new MultiObjSpec[mos.length];
        IdType[] idTypes = new IdType[mos.length];
        MultiObjSpec mo;
        int index = 0;

        // Copy mos[] elements into orderedMOS based
        //  on the order of elements in insertionOrder_
        for (int k = 0; k < insertionOrder_.length; k++) {
            Class objType = insertionOrder_[k];

            for (int j = 0; j < mos.length; j++) {
                if (objType != mos[j].getCls()) {
                    continue;
                }

                orderedMOS[index++] = mos[j];
            }
        }

        for (int j = 0; j < mos.length; j++) {
            idTypes[j] = new IdType(mos[j].obj.getObject().getId(), mos[j].getCls());
        }
        String calBlkId = getCalBlkId(idTypes);
        try {
            ObservingRun or = getObservingRun(mos);

            // Ensure the mode of the OR is ok.
            // Note: when P2PP is run in direct mode, this test
            // is in essence being performed twice, as the CheckInAction
            // class also checks the mode and instrument of the OB.
            if (!or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)
                && !(or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_VISITOR)
                    && Config.getCfg().isVMOBCheckinAllowed(or.getInstCode()))) {
                throw new ObjectIOException(
                    "Only objects for service-mode Observing Runs\n"
                        + "can be checked-in.");
            }
            if (!isAcceptablePeriod(or)) {
                throw new ObjectIOException(
                    "This OB/CB (for period "
                        + or.getPeriod()
                        + ") can not be checked-in"
                        + " as it is not in the acceptable set of check-in periods.");
            }
            ensureNotAlreadyCheckedIn(calBlkId);

            if (enforceObsRunIPVersionOnCheckIn) {
                ensureCorrectIPVersion(mos, or);
            }
        } catch (SQLException e) {
            throw new ObjectIOException(e);
        }

        multiStore(orderedMOS);
    }

    /**
     * Chech if the object already exists in the database - this is
     * done by checking if the Calblock/Obs Blk exists in the database.
     * @throws ObjectIOException if the object exists in the database.
     */
    private void ensureNotAlreadyCheckedIn(String calBlkId)
        throws ObjectIOException, SQLException {
        	
        Connection con = null;
        Statement stmt = null;

        try {
            con = acquireConnection();
            stmt = con.createStatement();

            String sql =
                "SELECT status FROM obs_blocks "
                    + " WHERE ob_id IN ("
                    + calBlkId
                    + ")";

            stdlog_.debug("ensureNotAlreadyCheckedIn: sql = " + sql);

            ResultSet rs = stmt.executeQuery(sql);

            if (rs.next()) {
                throw new ObjectIOException(
                    "This object is already checked-in!\n\n"
                        + "Please use the database browser to ensure that\n"
                        + "the checked-in items are as you wish them.\n"
                        + "\n"
                        + "If you have serious problems,"
                        + " contact ESO Support for help");
            }

        } finally {
            if (con != null) {
                try {
                    if (stmt != null) {
                        stmt.close();
                    }
                } finally {
                    releaseConnection(con);
                }
            }
        }
    }

    /**
     * Check all the CalibrationBlock objects, load their respective ObsRuns from
     * the database, and if the Cal Block ipversion does not match the version
     * specified in the loaded ObsRun, throw an ObjectIOException.
     */
    private void ensureCorrectIPVersion(MultiObjSpec[] mos, ObservingRun or)
        throws ObjectIOException, SQLException {

        float obsRunIPVersion = -1;
        float obIPVersion = -1;
        UnpackedStorableObject cBlock = getCalBlkUnpackedStorableObject(mos);

        if (cBlock == null) {
            throw new ObjectIOException("No OB or CB located in data.");
        }

        obsRunIPVersion = or.getIPVersion();

        // 1. load the Cal Blk/Obs Blk and get its ip version - remember that
        // the ipversion is stored in the Observation Description, not in the CB/OB.
        UnpackedStorableObject oduso = getODUnpackedStorableObject(mos);
        if (oduso == null) {
            throw new ObjectIOException("Observation Description not located in data.");
        }
        ObservationDescription od = (ObservationDescription)oduso.getObject();
        obIPVersion = od.getIPVersion();

        // 2. if versions do not match, throw and exception back to the user.

        if (obIPVersion != obsRunIPVersion) {
            throw new ObjectIOException(
                "The IP Version of the OB (v"
                    + obIPVersion
                    + ") does not match the IP Version expected for that Obs Run (expected v"
                    + obsRunIPVersion
                    + ")\n"
                    + "Please Download/Refresh your Observing Runs,\n"
                    + "open and save all OBs to be checked in,\n"
                    + "and then proceed with the Check-in.");
        }
    }

    /**
     * Throw an exception if the observing run contains a period that is not
     * in the acceptable period set. Id used when an OB is checked in through the
     * AppServer to limit the set of periods for which the appserver is to be used.
     *
     * @param or        containing the period number of the OB to be checked.
     * @return boolean  true if the OR period is in the acceptable period set.
     */
    private boolean isAcceptablePeriod(ObservingRun or) throws ObjectIOException {
        if (acceptablePeriods != null && acceptablePeriods.length > 0) {
            int period = or.getPeriod();
            for (int i = 0; i < acceptablePeriods.length; i++) {
                if (period == acceptablePeriods[i]) {
                    return true;
                }
            }
            // if we get here we have not found an accpetable period matching
            // the period of the observing run.
            return false;
        }
        return true;
    }

    /**
     * @return the ObservingRun for this OB/CB from the database.
     * @throws ObjectIOException if the ObservingRun is not found/invalid.
     */
    private ObservingRun getObservingRun(MultiObjSpec[] mos)
        throws ObjectIOException, SQLException {

        ObservingRun rc = null;
        UnpackedStorableObject cBlock = getCalBlkUnpackedStorableObject(mos);

        if (cBlock == null) {
            throw new ObjectIOException("No OB or CB located in data.");
        }
        // 1. get the obs run id.
        long orId = getObsRunId(cBlock);

        if (orId == -1) {
            throw new ObjectIOException("Invalid Observing Run Id: " + orId);
        }

        // 2. load the Obs Run from the database
        String error = "unknown error.";
        try {
            rc =
                (ObservingRun)OpcDbaseMgr.getInstance().getObj(
                    orId,
                    ObservingRun.class);
        } catch (ObjectNotFoundException e) {
            error = e.getMessage();
        }

        if (rc == null) {
            throw new ObjectIOException(
                "Observing Run for id: " + orId + " not found in database: " + error);
        }
        return rc;
    }

    /**
     * Check out the ObsBlk or CalBlk with the specified id plus
     * any attached objects, ie Target, ObsDescr, etc.
     *
     * @param   id      id of the ObsBlk or CalBlk to be checked out
     *
     * @return  CheckOutStatus object that contains a return
     *          status and (if successful) references to the
     *          checked-out objects.
     */
    public void checkOut(IdType[] idTypes)
        throws ObjectNotFoundException, ObjectIOException {
        IdType[] orderedIdTypes = new IdType[idTypes.length];
        int index = 0;

        // Copy mos[] elements into orderedMOS based
        //  on the order of elements in insertionOrder_
        for (int k = insertionOrder_.length - 1; k >= 0; k--) {
            Class objType = insertionOrder_[k];

            for (int j = 0; j < idTypes.length; j++) {
                if (objType != idTypes[j].objType) {
                    continue;
                }

                stdlog_.debug("checkOut: Will delete: " + idTypes[j]);
                orderedIdTypes[index++] = idTypes[j];
            }
        }

        try {
            checkStatus(idTypes);
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectIOException(e);
        }

        multiDelete(orderedIdTypes);
    }

    private void checkStatus(IdType[] idTypes)
        throws ObjectNotFoundException, ObjectIOException, SQLException {
        Connection con = null;
        Statement stmt = null;

        try {
            con = acquireConnection();
            stmt = con.createStatement();

            String sql =
                "SELECT status FROM obs_blocks "
                    + " WHERE ob_id IN ("
                    + getCalBlkId(idTypes)
                    + ")";

            stdlog_.debug("checkStatus: sql = " + sql);

            ResultSet rs = stmt.executeQuery(sql);

            if (!rs.next()) {
                throw new ObjectNotFoundException("Objects not found in database");
            }

            do {
                String status = rs.getString("status");

                if (!CalibrationBlock.DEFINED.equals(status)
                    && !CalibrationBlock.REJECTED.equals(status)) {
                    throw new ObjectIOException(
                        "This object cannot now be checked out.\n"
                            + "It has been passed to the telescope team"
                            + " for execution at a later date.\n"
                            + "status = "
                            + status);
                }
            } while (rs.next());
        } finally {
            if (con != null) {
                try {
                    if (stmt != null) {
                        stmt.close();
                    }
                } finally {
                    releaseConnection(con);
                }
            }
        }
    }

    /**
     * @return the dbase id for the calibration block in the idType
     * array passed. This assumes there is exactly ONE Calib or Obs Block
     * in the array.
     */
    private String getCalBlkId(IdType[] idTypes) {

        StringBuffer rc = new StringBuffer();
        for (int j = 0; j < idTypes.length; j++) {
            if (idTypes[j].objType == CalibrationBlock.class
                || idTypes[j].objType == ObservationBlock.class) {
                Long idLong = new Long(idTypes[j].id);
                rc.append("" + idLong);
                // remove the last two numbers in the id
                // effectively converting it to the database id
                // This should be reworked to use uniqueToTable()
                if (rc.length() > 0) {
                    rc.setLength(rc.length() - 2);
                }
                return rc.toString();
            }
        }

        return rc.toString();
    }

    /**
     * @return the UnpackedStorableObject from the multi object spec array for
     * the cal block, or null if not found.
     */
    private UnpackedStorableObject getCalBlkUnpackedStorableObject(MultiObjSpec mos[]) {

        UnpackedStorableObject rc = null;

        for (int j = 0; rc == null && j < mos.length; j++) {
            if (mos[j].getCls() == CalibrationBlock.class
                || mos[j].getCls() == ObservationBlock.class) {
                rc = mos[j].obj;
            }
        }

        return rc;
    }

    /**
     * @return the UnpackedStorableObject from the multi object spec array for
     * the observation description, or null if not found.
     */
    private UnpackedStorableObject getODUnpackedStorableObject(MultiObjSpec mos[]) {

        UnpackedStorableObject rc = null;

        for (int j = 0; rc == null && j < mos.length; j++) {
            if (mos[j].getCls() == ObservationDescription.class) {
                rc = mos[j].obj;
            }
        }

        return rc;
    }

    /**
     * @return the id of the ObsRun from the UnpackedStorableObject
     * If the cBlk is not a CalibrationBlock or ObservationBlock,
     * -1 is returned.
     */
    private long getObsRunId(UnpackedStorableObject cBlk) {
        long rc;
        if (cBlk.getObject() instanceof CalibrationBlock) {
            //rc = Config.getCfg().uniqueToTableId(cBlk.getNestedObjId("ObsRun"));
            rc = cBlk.getNestedObjId("ObsRun");
        } else {
            rc = -1;
        }
        return rc;
    }

    /**
     * Returns a string representation of this object and its fields.
     *
     * @return a string representation.
     */
    public String toString() {
        return "DbaseMgr[rootNode=" + getRoot() + "]";
    }

    protected static class Phase2HandlerRegistry extends HandlerRegistry {

        /**
         * Initialises the registry with handler classes for every object type.
         */
        protected void initialiseRegistry() {
            super.initialiseRegistry();
            stdlog_.debug("dbase.phase2.DbaseMgr initialiseRegistry");
            register(new DbaseHandlerCB());
            register(new DbaseHandlerOB());
            register(new DbaseHandlerTG());
            register(new DbaseHandlerOD());
            register(new DbaseHandlerCS());
            register(new DbaseHandlerQueue());
            register(new DbaseHandlerMaskQueue());
            register(new DbaseHandlerMask());
        }

    }

    /**
     * Returns the acceptablePeriods.
     * @return int[]
     */
    public int[] getAcceptablePeriods() {
        return acceptablePeriods;
    }

    /**
     * Sets the acceptablePeriods.
     * @param acceptablePeriods The acceptablePeriods to set
     */
    public void setAcceptablePeriods(int[] acceptablePeriods) {
        this.acceptablePeriods = acceptablePeriods;
    }



}

