package org.eso.ohs.phase2.visiplot;

/**
 * JFrame extension for displaying, saving and printing plots.
 *
 * There are two methods that setup up the chart panels - one is for display
 * and the other is for printing. Modifications to the plots need to be made
 * in both of these methods:
 *
 *  - displayPlots()
 *  - setupPrintPanel()/disposePrintPanel();
 *
 * Note that the JTabbedPane is updated dynaically as tab panes are selected.
 * This is done to reduce the memory used displaying plaits. 
 * The JPanel objects used to render the plots are stored in the Vector
 * memeber chartPanels_. These panels are only populated once they are
 * displayed, and cleared when no longer needed.
 *
 * @version $Id: ChartFrame.java,v 1.31 2005/12/01 15:53:31 pnunes Exp $
 * @author Nick Kornweibel for ESO
 * <br>ChangeLog: cmavros 29May02 [1721/OT] Add progressbar before generating plot. 
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.PrintJob;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.Random;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.visibility.AirMass;
import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCFInterval;

public class ChartFrame extends JFrame {

    /**
     * Standard log channel
     * <br>ChangeLog: cmavros 21-12-01 added
     */
    private static Logger stdlog_ = Logger.getLogger(ChartFrame.class);

    private Vector vplots_ = null;
    private Vector reports_ = null;
    private VisiPlotOptions currentSettings_;
    
    private JPanel chartFramePanel_;
    private JTabbedPane tabbedPane_;
    private Vector chartPanels_ = new Vector();



    // used to select file names for saving plots as GIF files.
    // should be accessed through the getFileChooser() method.
    private JFileChooser fileChooser_ = null;

    // Members for printing:
    private int pageBorder_ = 30;
    private int plotsPerPage_ = 10;
    private JFrame printFrame_ = null;
    private JPanel printPanel_ = null;

    private Font displayFont_ = new Font("Helvetica", Font.PLAIN, 12);
    // private Font printerFont_ = new Font("Courier", Font.PLAIN, 10);
    private Font printerFont_ = displayFont_;

    private int chartDisplayWidth_ = 700;
    private int chartDisplayHeight_ = 90;
    private int constraintChartDisplayHeight_ = 15;

    private int paneDisplayWidth_ = 700;
    private int paneDisplayHeight_ = 400;

    private ObjectContainer folderView_;
	private ChartColorItem[] chartColorItems;
	protected Long [] idLongs_;



       /**
     * Initiliaze the ChartFrame with a set of ObservationBlock
     * 
     * @deprecated Use the other constructor. Visiplot needs the settings
     */

    public ChartFrame( ObjectContainer folderView) {
        buildFrame(folderView);
    }
    /**
     * Initialize the ChartFrame with a set of ObservationBlock
     */

    public ChartFrame( ObjectContainer folderView,VisiPlotOptions setting, ChartColorItem[] chartColorItems , Long [] ids) {
        currentSettings_ = setting;
        this.chartColorItems = chartColorItems;
        idLongs_ = ids;
        buildFrame(folderView);
        
    }

	/**
     * @param obs
     * @param folderView void
     */
    private void buildFrame(ObjectContainer folderView) {
    	     
        folderView_ = folderView;
        setupMenu();
        chartFramePanel_ = new JPanel();
        tabbedPane_ = new JTabbedPane();
        tabbedPane_.addChangeListener(new PanelBuilder());
        chartFramePanel_.add(tabbedPane_);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeChartFrame();
            }
        });
        tabbedPane_.setPreferredSize(
            new Dimension(paneDisplayWidth_, paneDisplayHeight_));
        setTitle("Visibility Plots");
        getContentPane().add(tabbedPane_);
    }

    /**
     * Set up the Frame menu.
     */

    private void setupMenu() {
        JMenu menu = new JMenu("File");
        JMenuItem item = null;
        // edit
        item = new JMenuItem("edit...");
                        
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	                
                if (showDialog(true)) {
                	generateProgressBar(folderView_,currentSettings_,idLongs_);
			    
			    }                                
            }
        });
        
        if (idLongs_ != null)
        	menu.add(item);

        // print
        item = new JMenuItem("print...");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                printPlots();
            }
        });
        menu.add(item);

        // save
        item = new JMenuItem("save to GIF...");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveToFile("gif");
            }
        });
        menu.add(item);

        // close
        item = new JMenuItem("close");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeChartFrame();
            }
        });
        menu.add(item);

        JMenuBar menuBar = new JMenuBar();
        menuBar.add(menu);
        setJMenuBar(menuBar);
    }

    /**
     * Close the chart frame and dispose of its components.
     */

    public void closeChartFrame() {
        setVisible(false);
        dispose();
    }

    /**
     * Utility method for displaying a "not implemented" message.
     */

    private void showNotImplemented(String msg) {
        JOptionPane.showMessageDialog(
            this,
            new Object[] { "Sorry! Not implemented" },
            msg,
            JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Simply utility method to preserve a single JFileChooser for use
     * in all ChartFrames.
     */

    private JFileChooser getFileChooser() {
        if (fileChooser_ == null) {
            fileChooser_ = new SwingFileChooser();
        }
        return fileChooser_;
    }

    /**
     * See note on header as to how the tabbed pane is populated.
     */

    protected JPanel addTabPanel(String title) {
        JPanel chartPanel = new JPanel();
        chartPanel.setLayout(new GridLayout(0, 1));
        // add panel the Vector of panels.
        chartPanels_.addElement(chartPanel);
        JScrollPane chartPanelScroller = new JScrollPane(chartPanel);
        chartPanelScroller.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        chartPanelScroller.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        tabbedPane_.add(title, chartPanelScroller);
        return chartPanel;
    }

    /**
     * Display calculated visibility plots.
     * Does nothing if the plots have not been calculated.
     */

    public void displayPlots() {
        if (vplots_ != null) {
            setVisible(false);
            tabbedPane_.removeAll();
            chartPanels_.removeAllElements();
            int numTabs = (vplots_.size() / plotsPerPage_);
            if (vplots_.size() % plotsPerPage_ > 0) {
                numTabs++;
            }
            for (int i = 0; i < numTabs; i++) {
                int startIdx = (i * 10) + 1;
                int endIdx = (i * 10) + plotsPerPage_;
                if (endIdx > vplots_.size()) {
                    endIdx = vplots_.size();
                }
                String title;
                if (startIdx == endIdx) {
                    title = "" + startIdx;
                } else {
                    title = "" + startIdx + "-" + endIdx;
                }
                addTabPanel(title);
            }
            chartFramePanel_.doLayout();
            invalidate();
            setVisible(true);
        }
    }

    /**
     * Populate a panel with plots from start to end (inclusive).
     * Assumes all indexes are with in the range of available data.
     * @param currentTab the JPanel to populate
     * @param start the start index of plot data (from 0).
     * @param end the index of the last plot to be added to the panel.
     */

    protected void populatePanel(JPanel currentTab, int start, int end) {
        if (vplots_ != null) {
            for (int i = start; i <= end; i++) {
                OBVisiplotInfo visi = (OBVisiplotInfo) vplots_.elementAt(i);

                // base panel for all reports, plots, etc.
                JPanel visiPlotPanel =
                    createPlotPanel(
                        displayFont_,
                        visi,
                        (String[]) reports_.elementAt(i));

                // add the base panel to this scrolling panel.
                currentTab.add(visiPlotPanel);
            }
            currentTab.doLayout();
        }
    }

    /**
     * Setup the frame and panel for printing - it is only used when
     * the print menu option has been selected.
     * Note that calling setupPrintPanel for large numbers of plots (> 50)
     * may will cause an Xterminal to run out of graphic memory, resulting
     * in display/rendering problems with the plots.
     * @return array of JPanels to be printed.
     */

    private JPanel[] setupPrintPanel(int width, int height) {
        return setupPrintPanel(width, height, 0, vplots_.size() - 1);
    }

    /**
     * Setup the frame and panel for printing - it is only used when
     * the print menu option has been selected.
     * Note that calling setupPrintPanel for large numbers of plots (> 50)
     * may will cause an Xterminal to run out of graphic memory, resulting
     * in display/rendering problems with the plots.
     * @param startIdx - the first plot to print (start at 0).
     * @param endIdx - the plot to print (max vplots_.size()-1).
     * @return array of JPanels to be printed.
     */

    private JPanel[] setupPrintPanel(
        int width,
        int height,
        int startIdx,
        int endIdx) {

        // dispose old print panel...to be sure, to be sure.
        disposePrintPanel();
        // ensure the indexes are acceptable, range-wise.
        if (startIdx < 0) {
            startIdx = 0;
        }
        if (endIdx >= vplots_.size()) {
            endIdx = vplots_.size() - 1;
        }

        JPanel rc[] = new JPanel[endIdx - startIdx + 1];
        printPanel_ = new JPanel();
        printFrame_ = new JFrame();
        printPanel_.setBackground(Color.white);
        printPanel_.setLayout(new GridLayout(0, 1));
        printFrame_.getContentPane().add(printPanel_);

        for (int i = startIdx; i <= endIdx; i++) {
        	OBVisiplotInfo visi = (OBVisiplotInfo) vplots_.elementAt(i);

            // create the base panel
            JPanel visiPlotPanel =
                createPlotPanel(
                    printerFont_,
                    visi,
                    (String[]) reports_.elementAt(i));

            // add to the panel to the report panel.
            printPanel_.add(visiPlotPanel);
            rc[i - startIdx] = visiPlotPanel;
        }
        printPanel_.doLayout();
        printFrame_.doLayout();
        printFrame_.pack();
        printFrame_.invalidate();
        return rc;
    }

    private void disposePrintPanel() {
        if (printPanel_ != null) {
            printPanel_.removeAll();
        }
        if (printFrame_ != null) {
            printFrame_.dispose();
        }
        printPanel_ = null;
        printFrame_ = null;
    }

    /**
     * Create a panel with all the constraint plots, text report
     * and visibility plot as required.
     * Used for both display and printing purposes.
     */

    protected JPanel createPlotPanel(
        Font font,
        OBVisiplotInfo visi,
        String[] report) {
        // base panel for all reports, plots, etc.
        JPanel visiPlotPanel = new JPanel(new BorderLayout());

        // Create the contraints set panel and add it.
        // setting num rows 0 for GridLayout - this forces the spec.
        // column count (1) to be maintained.
        JPanel constraintsPanel = new JPanel(new GridLayout(0, 1));

        // create the required constraint charts and add them to the
        // constraintsPanel.

        PCF constraint = null;

        constraint = visi.getAirMassPCF();
        if (visi.isAirMassEnabled() && constraint != null) {
            // The airmass PCF needs to be created anew as we
            // simply display the value 1 when it is non-zero, else
            // display 0.
            PCF binaryAirMass = new PCF();
            for (int x = 0; x < constraint.getIntervalCount(); x++) {
                PCFInterval interval = constraint.getIntervalAt(x);
                double val = 0.0;
                if (interval.getValue() > 0.0
                    && interval.getValue() < AirMass.NOT_VIS) {
                    val = 1.0;
                }
                binaryAirMass.addInterval(
                    interval.getStart(),
                    interval.getEnd(),
                    val);
            }
            assert (chartColorItems != null): MsgManager.errInvariant("chartColorItems array is null");
            assert (chartColorItems.length > 0): MsgManager.errInvariant("chartColorItems array is empty");
            
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "air mass",
                    binaryAirMass,
                    visi.getPrecision(),
                    chartColorItems[0]);
            constraintsPanel.add(constraintChart);
        }
        constraint = visi.getSunPCF();
        if (constraint != null && constraint.getIntervalCount() > 1) {
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "sun",
                    constraint,
                    visi.getPrecision(),
                    chartColorItems[3]);
            constraintsPanel.add(constraintChart);
        }

        constraint = visi.getFLIPCF();
        if (constraint != null && constraint.getIntervalCount() > 1) {
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "fli",
                    constraint,
                    visi.getMoonUpPCF(),
                    visi.getPrecision(),
                    chartColorItems[2]);
            constraintsPanel.add(constraintChart);
        }

        constraint = visi.getMoonDistPCF();
        if (constraint != null && constraint.getIntervalCount() > 1) {
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "moon dist",
                    constraint,
                    visi.getMoonUpPCF(),
                    visi.getPrecision(),
                    chartColorItems[1]);
            constraintsPanel.add(constraintChart);
        }
        
        
        constraint = visi.getTimeIntervalPCF();
        if (constraint != null && constraint.getIntervalCount() > 1) {
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "time int's",
                    constraint,
                    visi.getPrecision(),
                    chartColorItems[4]);
            constraintsPanel.add(constraintChart);
        }
        

        constraint = visi.getSiderealTimeIntervalPCF();
        if (constraint != null && constraint.getIntervalCount() > 1) {
            ChartPanel constraintChart =
                getConstraintChart(
                    chartDisplayWidth_,
                    constraintChartDisplayHeight_,
                    font,
                    "st time int's",
                    constraint,
                    visi.getPrecision(),
                    chartColorItems[6]);
            constraintsPanel.add(constraintChart);
        }
        // create the visibility plot panel and add it.
        ChartPanel chart =
            getVisibilityChart(
                chartDisplayWidth_,
                chartDisplayHeight_,
                font,
                visi,
                chartColorItems[5]);

        // create the text report panel and add it.
        JPanel textReport = getTextReportPanel(font, report);

        // Place report and constraint plots in the north panel
        // of the visiPlotPanel and the chart in the center,
        // allowing the chart to strech to fill the scroll pane
        // is required.
        JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.add(textReport, BorderLayout.NORTH);
        northPanel.add(constraintsPanel, BorderLayout.SOUTH);
        visiPlotPanel.add(northPanel, BorderLayout.NORTH);
        visiPlotPanel.add(chart, BorderLayout.CENTER);
        return visiPlotPanel;
    }
    

    /**
     * Build the plot panels and print then, plotsPerPage_ per page.
     * Does nothing if the plots have not been calculated.
     * Plots are generated on a per page basis, with plotsPerPage_ plots
     * on each page. The plots for a page are generate with setupPrintPanel()
     * and disposed of when printed with disposePrintPanel().
     */

    private void printPlots() {
        if (vplots_ != null) {
            Properties p = new Properties();
            // the following properties apply only to UNIX java VM.
            // need to put these in general properties box.
            p.put("awt.print.numCopies", "1");
            p.put("awt.print.paperSize", "a4");
            p.put("awt.print.destination", "printer");
            p.put("awt.print.orientation", "portrait");
            p.put(
                "awt.print.printer",
                currentSettings_.getValue(VisiPlotOptions.printerName));

            PrintJob pjob = getToolkit().getPrintJob(this, "visiplot", p);
            if (pjob != null) {
                Cursor currentCursor = getCursor();
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                Graphics pg = pjob.getGraphics();
                if (pg != null) {

                    // set up widths, heights, etc.
                    Dimension page = pjob.getPageDimension();
                    int plotWidth = page.width - (pageBorder_ * 2);
                    int plotHeight =
                        (page.height - (pageBorder_ * 2)) / plotsPerPage_;

                    // loop through the plots printing each one.
                    boolean disposeMe = false;
                    JPanel[] plots = null;
                    int numPlots = vplots_.size();

                    for (int i = 0; i < numPlots; i++) {

                        // request a batch of plots if required.
                        if (i % plotsPerPage_ == 0) {
                            int startIdx = i;
                            int endIdx = i + plotsPerPage_ - 1;
                            if (endIdx >= numPlots) {
                                endIdx = numPlots - 1;
                            }
                            plots =
                                setupPrintPanel(
                                    plotWidth,
                                    plotHeight,
                                    startIdx,
                                    endIdx);
                        }

                        // request a new print page if required.
                        if (pg == null) {
                            pg = pjob.getGraphics();
                            if (pg == null) {
                                // oops - stop printing!
                                break;
                            }
                        }

                        stdlog_.info(
                            "Printing plot: " + (i + 1) + " of " + numPlots);

                        // print a plot.
                        Image chart =
                            ChartUtils.getImageFromComponent(
                                plots[i % plotsPerPage_]);
                        pg.drawImage(
                            chart,
                            pageBorder_,
                            pageBorder_ + ((i % plotsPerPage_) * plotHeight),
                            plotWidth,
                            plotHeight,
                            plots[i % plotsPerPage_]);
                        disposeMe = true;

                        // clean up ready for a new page if required.
                        if ((i + 1) % plotsPerPage_ == 0) {
                            // end of page - send it to the printer.
                            pg.dispose();
                            pg = null;
                            disposeMe = false;
                            // remove the generated plots ready to get new ones.
                            disposePrintPanel();

                        }
                    } // end of for loop.
                    if (disposeMe) {
                        pg.dispose();
                    }
                }
                // end the print job.
                stdlog_.info("Ending print job");
                pjob.end();
                // remove the generated plots ready to get new ones.
                disposePrintPanel();
                stdlog_.info("Print job ended");
                setCursor(currentCursor);
            }
        }
    }

    /**
     * Save to file. Only if the plots have been calculated.
     * @param the default extension to be applies to the selected file.
     */

    private void saveToFile(String ext) {
        if (vplots_ != null) {
            JFileChooser fc = getFileChooser();
            fc.setDialogTitle("Select file path to save to");
            fc.showSaveDialog(this);
            File target = fc.getSelectedFile();
            if (target != null) {
                Cursor currentCursor = getCursor();
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                String targetBase = target.getAbsolutePath();
                ext = "." + ext;
                int numFiles = ((vplots_.size() - 1) / plotsPerPage_) + 1;
                if (numFiles == 1) {
                    // only one file - output to the file selected.
                    if (ext != null && ext.length() != 0) {
                        ext = ext.toLowerCase();
                        if (!target.getName().toLowerCase().endsWith(ext)) {
                            target = new File(target.getPath() + ext);
                        }
                    }
                    saveToFile(target, 0, vplots_.size() - 1);
                    setCursor(currentCursor);
                    JOptionPane.showMessageDialog(
                        this,
                        new Object[] { "Created the following file:", target },
                        "Visplot save to GIF file",
                        JOptionPane.INFORMATION_MESSAGE);
                } else {
                    // multiple files - print the files each with an
                    // index extension
                    // if user included extension...remove it to get the
                    // base name.
                    if (targetBase.endsWith(ext)) {
                        targetBase =
                            targetBase.substring(
                                0,
                                targetBase.length() - ext.length());
                    }
                    String[] fileNames = new String[numFiles + 1];
                    fileNames[0] =
                        "Plots will be saved "
                            + plotsPerPage_
                            + " per file to the following files:";
                    for (int i = 1; i <= numFiles; i++) {
                        fileNames[i] = targetBase + "_" + i + ext;
                    }
                    JOptionPane.showMessageDialog(
                        this,
                        fileNames,
                        "Saving plots as GIF files",
                        JOptionPane.INFORMATION_MESSAGE);
                    for (int i = 0; i < numFiles; i++) {
                        // NB: skip fileNames[0] - it is a part of the message.
                        String file = fileNames[i + 1];
                        int startIdx = i * plotsPerPage_;
                        int endIdx = startIdx + plotsPerPage_ - 1;
                        if (endIdx >= vplots_.size()) {
                            endIdx = vplots_.size() - 1;
                        }
                        saveToFile(new File(file), startIdx, endIdx);
                    }
                    setCursor(currentCursor);
                }
            }
        } // end of if statement.
    }

    private void saveToFile(File target, int startIdx, int endIdx) {
        try {
            stdlog_.info("Generating file: " + target.getAbsolutePath());
            setupPrintPanel(
                chartDisplayWidth_,
                chartDisplayHeight_,
                startIdx,
                endIdx);
            ChartUtils.saveComponentToGIFFile(target, (JComponent) printPanel_);
        } catch (IOException ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(
                this,
                new Object[] { "Error saving GIF file" },
                ex.toString(),
                JOptionPane.ERROR_MESSAGE);
        } finally {
            disposePrintPanel();
        }
    }

    /**
     * Set the VisiPlotOptions object to be used for the calulations
     * and in the settings dialog.
     */

    public void setVisiPlotOptions(VisiPlotOptions opts) {
        currentSettings_ = opts;
    }

    /**
     * Display the options dialog, using the specified Frame as the 
     * dialog parent.
     * @return true if the user wishes to proceed with the plot generation
     * else false.
     */

    
    
    





    /**
     * Randomly generated PCF for visibility plot display
     * This is included for debugging/testing purposes.
     * @return the random PCF, or null if the creation failed.
     */

    private PCF getRandomPCF() {
        PCF pcf = new PCF();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date startDay = null;
        try {
            startDay = formatter.parse("2001-01-01");
        } catch (ParseException e) {
            // will this happen? anyway, set pcf tp null.
            pcf = null;
        }
        if (pcf != null) {
            Random seed = new Random();
            long now = startDay.getTime() + ((22 * 60 * 60) * 1000);
            long hour = (60 * 60 * 1000);

            pcf.addInterval(
                new Date(now),
                new Date(now + hour),
                seed.nextDouble());
            now = now + hour;
            double lastValue = seed.nextDouble();
            for (int x = 0; x < 12; x++) {
                now = now + hour;
                if (seed.nextDouble() < 0.3) {
                    // change the value 30% of the time.
                    lastValue = seed.nextDouble();
                }
                pcf.appendInterval(new Date(now), lastValue);
            }
        }
        return pcf;
    }

    /**
     * @return a JPanel object containing the text report
     * for the String array passed.
     */

    private JPanel getTextReportPanel(Font f, String[] report) {
        JPanel rc = new JPanel(new GridLayout(0, 1));
        rc.setBackground(Color.white);
        for (int l = 0; l < report.length; l++) {
            JLabel line = new JLabel(report[l]);
            line.setFont(f);
            rc.add(line);
        }
        return rc;
    }

    /**
     * @return a visibility chart with the specified width,
     * height and font, using the specified PCF object.
     */

    private ChartPanel getVisibilityChart(
        int width,
        int height,
        Font f,
        OBVisiplotInfo visi, ChartColorItem item) {
        VisibilityChart vChart = new VisibilityChart(visi, item);
        ChartPanel rc = new ChartPanel(vChart);
        rc.setChartSize(width, height);
        rc.setAllFonts(f);
        rc.setPlotPosition(0.2, 0.5, 0.9, 0.8);
        return rc;
    }

    /**
     * 
     * @return a ContraintChart panel with the given name
     * and dimensions and based on the data passed in the PCF.
     */

    private ChartPanel getConstraintChart(
        int width,
        int height,
        Font f,
        String name,
        PCF pcf,
        int precision, ChartColorItem item) {
        return getConstraintChart(width, height, f, name, pcf, null, precision, item);
    }

    /**
     * Build a ConstraintChart. The altColorPcf controls the color of the chart.
     * If null, the chart if gray, if non-null, values in the altColorPcf that
     * are > 0.0 will be rendered in darkGray, the rest in gray.
     * @param width
     * @param height
     * @param f
     * @param name
     * @param pcf
     * @param altColorControl
     * @param precision
     * @return ChartPanel
     */
    private ChartPanel getConstraintChart(
        int width,
        int height,
        Font f,
        String name,
        PCF pcf,
        PCF altColorControl,
        int precision, ChartColorItem item) {
    	
    	
    	
        ConstraintChart cChart =
            new ConstraintChart(pcf, altColorControl, precision, name, item);
        ChartPanel rc = new ChartPanel(cChart);
       cChart.setForeground(Color.red);
       
        rc.setChartSize(width, height);
        rc.setAllFonts(f);
        //rc.setPlotPosition(0.2, 0.05, 0.9, 0.95);
        rc.setPlotPosition(0.2, 0.1, 0.9, 0.2);
        return rc;
    }


    public boolean showDialog(Frame owner, boolean modal) {
    	// build the dialog to show the plotting options
        VPlotOptionsDialog dlg = new VPlotOptionsDialog(owner, modal,false);        
        dlg.showDialog(currentSettings_);
        chartColorItems = dlg.getChartColorItems();
        return dlg.getSettingsAccepted();
    }
    
    private boolean showDialog(boolean modal) {
        return showDialog((JFrame) folderView_.getTopLevelAncestor(), modal);
    }


	/**
     * Load the OB and Calculate the visibility with a progress bar
     */
    private void generateProgressBar(ObjectContainer parent, VisiPlotOptions visiOptions, Long [] obIds ) {
        new IncrementalActionAutomaton(
            new IncrementalOBLoadingVisiCalc(visiOptions, obIds,new ChartFrame(folderView_,visiOptions,chartColorItems,obIds)),
            (JFrame) parent.getTopLevelAncestor(),
            "Calculating ...");
    }

   
    /**
     * Adds an OB report to the report vector
     * @param report
     */
    public void addReport(String report[]) {
    	if (reports_ == null)
    		reports_ = new Vector();
    	reports_.addElement(report);
    }
    
    /**
     * Adds an computed OB to the vplots vector
     * @param visi
     */
    public void addVisiplot(OBVisiplotInfo visi) {
    	if (vplots_ == null)
    		vplots_ = new Vector();
    	vplots_.addElement(visi);
    }
    

    class PanelBuilder implements ChangeListener {
        private int lastIdx = -1;
        public void stateChanged(ChangeEvent evt) {
            int i = tabbedPane_.getSelectedIndex();

            // clean up last displayed panel.
            if (lastIdx >= 0 && lastIdx < chartPanels_.size()) {
                JPanel chartPanel = (JPanel) chartPanels_.elementAt(lastIdx);
                chartPanel.removeAll();
            }
            lastIdx = i;

            // display the newly selected panel.
            if (i >= 0) {
                int startIdx = i * 10;
                int endIdx = (i * 10) + plotsPerPage_;
                if (endIdx > vplots_.size()) {
                    endIdx = vplots_.size();
                }
                JPanel chartPanel = (JPanel) chartPanels_.elementAt(i);
                populatePanel(chartPanel, startIdx, endIdx - 1);
            }
        }
    }
}
