package org.eso.ohs.phase2.visiplot;

/**
 * Extension of the BarChart bean used to display Visibility data from
 * a VisiPlot object.
 *
 * A Small note about how the visibility graph is built, over coming some
 * limitations of KavaChart. An email has been sent the KavaChart asking
 * for other means of achieving this result....no news yet.
 *
 * Background:
 *   - The Y axis of graphs (at least for KavaChart bar graphs) must
 *   increase as it goes UP the page...that is, the Y axis value 2.5 must
 *   be above 1.5...etc - there is not changing this.
 *   - Good visilibity AirMass is 1.0
 *   - Poor visibility (for VLT) is 4.0 (this is not hard coded, but defined
 *   as AirMass.NOT_VIS, in AirMass.java.
 * 
 * Our requirement:
 *   We need a bar graph with a Y axis where 1.0 is at the top (good
 *   visibility airmass) and 4.0 is at the bottom (poor visibility airmass).
 * 
 * The solution:
 *   - The visiblity PCF is first inverted. That is, all values are multiplied
 *   by -1.0. This effectively gives us the same shape, though up side down.
 *   - The Y Axis Axis class has been extended with the NegativeToPositiveAxis
 *   (defined in this file). This class does little else but remove the -ve signs
 *   from the normal Axis labels.
 *   - Finally, the base line of the graph is reset to -4.0, thus inverting
 *   the appearance of the graph.
 *
 *   The final result is a bar graph that appears to range from 4.0 to 1.0,
 *   with bars extending from the X axis (4.0) up to their air mass values.
 *
 * @author Nick Kornweibel for ESO
 */

import javachart.beans.chart.BarChart;
import javachart.chart.Axis;
import javachart.chart.Bar;
import javachart.chart.Dataset;

import org.eso.ohs.phase2.visibility.AirMass;
import org.eso.ohs.phase2.visibility.PCF2Dataset;
import org.eso.ohs.phase2.visibility.PCFDataFeed;

public class VisibilityChart extends BarChart {

    /**
     * Create a chart from the given VisiPlot.
     * @param visi The VisiPlot containing the set of PCFs used to 
     * create the chart.
     */

    public VisibilityChart(OBVisiplotInfo visi, ChartColorItem colorItem) {
        int precision =  visi.getPrecision();
         
        // note the PCF values are multiplied by -1.0...
        // See the header of this class for the reason why this is required.
        PCFDataFeed dataBean
          = new PCFDataFeed(VisiplotUtils.setToNegative(visi.getResultPCF()), precision);

        Dataset execTimeData = null;                
        if (visi.getExecTimePCF() != null) {
            // user wants to display exec time graphically
            execTimeData = PCF2Dataset.convertPCF2Dataset(
                visi.getExecTimePCF(), precision);
        }
        String labels[]
            = VisiplotUtils.createLabels(visi.getStartTime(), visi.getEndTime(), precision);
        init(dataBean, execTimeData, labels, colorItem);
    }

    /**
     * Initialize all graph settings - called from constructor.
     * @param data feed bean containing the data from the PCF
     */

    private void init(PCFDataFeed dataFeed, Dataset execTime,
            String[] labels, ChartColorItem colorItem) {
        dataFeed.addDataFeedListener(this);
        dataFeed.update();

        javachart.chart.BarChart chart
            = (javachart.chart.BarChart)getChart();

        // set the colors for the graph to be rendered.
        chart.getDatasets()[0].getGc().setFillColor(colorItem.getAreaColor());

        if (execTime != null) {
            // we want to indicate execution time...
            execTime.getGc().setFillColor(colorItem.getPerimeterColor());

            // over-ride the standard Bar with our ColorChangeBar.
            Bar b = new ColorChangeBar(chart.getDatasets(), execTime,
                chart.getXAxis(), chart.getYAxis(), chart.getPlotarea());
            chart.setBar(b);
        }

        // remove the title.
        chart.getBackground().setTitleString("");

        // The baseline is the NOT_VIS limit set in AirMass - except
        // we need the -ve of that number here...
        // See the header of this class for the reason why this is required.
        chart.getBar().setBaseline(AirMass.NOT_VIS*(-1.0));
        chart.getBar().setClusterWidth(1.0);

        // Y Axis
        chart.setYAxis(new NegativeToPositiveAxis());
        chart.getYAxis().setLabelPrecision(1);
        chart.getYAxis().setMajTickLength(2);
        chart.getYAxis().setTitleString("air mass");

        // X Axis
        chart.getXAxis().setMajTickLength(2);
        chart.getXAxis().addLabels(labels);
        chart.getXAxis().setTitleString("UT");
    }

    /**
     * An Axis specialization that simply removes the -ve signs from the
     * labels prior to returning them.
     * See the header of this class for the reason why this is required.
     */

    class NegativeToPositiveAxis extends Axis {
        public String getLabel(double inVal, int count) {
            String rc = super.getLabel(inVal, count);
            return rc.replace('-', ' ');
        }
    }
}
