package org.eso.ohs.core.utilities;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.commons.collections.CollectionUtils;

/**
 * Static helper class that tries to identify the file format for a given file
 * or byte array representing the first bytes of a file.
 * <h3>Usage</h3>
 * 
 * <pre>
 * FormatDescription desc = FormatIdentification
 * 		.identify(new File(&quot;testfile.zip&quot;));
 * if (desc != null) {
 * 	System.out.println(desc.getShortName());
 * }
 * </pre>
 * 
 * @author Marco Schmidt
 */
public class FormatIdentification {
	private List descriptions;

	private int minBufferSize;

	private static FormatIdentification format = null;

	

	/**
	 * Private empty constructor to avoid instantiations. *
	 */
	private FormatIdentification() {
		descriptions = new ArrayList();
		init();
	}

	/**
	 * Private empty constructor to avoid instantiations. *
	 */
	private FormatIdentification(InputStreamReader is) {
		try {
			descriptions = new ArrayList();
			init(is);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * singleton. *
	 */
	public static FormatIdentification getInstance() {
		if (format == null) {
			format = new FormatIdentification();
		}
		return format;
	}

	/**
	 * singleton. *
	 */
	public static FormatIdentification getInstance(InputStreamReader is) {
		if (format == null) {
			format = new FormatIdentification(is);
		}
		return format;
	}

	public FormatDescription identify(byte[] data) {
		if (data == null || data.length < 1) {
			System.out
					.println("----------------------------- Format Description: data is null or < 1");
			return null;
		}

		List localDescription = new ArrayList();

		synchronized (descriptions) {
			Iterator iter = descriptions.iterator();
			CollectionUtils.addAll(localDescription, iter);
		}

		Iterator localIterator = localDescription.iterator();
		while (localIterator.hasNext()) {
			FormatDescription desc = (FormatDescription) localIterator.next();
			//System.out
			//		.println("----------------------------- Format Description: name
			// is : "
			//				+ desc.getLongName());
			if (desc.matches(data)) {
				return desc;
			}
		}
		//System.out
		//		.println("----------------------------- Format Description: no match
		// found");
		return null;
	}

	public FormatDescription identify(File file) {
		if (!file.isFile()) {
			//System.out
			//		.println("----------------------------- Format Description: file
			// "
			//				+ file.getAbsolutePath() + " is not a file");
			return null;
		}
		long size = file.length();
		int numBytes;
		if (size > minBufferSize) {
			numBytes = minBufferSize;
		} else {
			numBytes = (int) size;
		}
		byte[] data = new byte[numBytes];
		RandomAccessFile in = null;
		try {
			in = new RandomAccessFile(file, "r");
			in.readFully(data);
			in.close();
		} catch (IOException ioe) {
			//System.out
			//		.println("----------------------------- Format Description: file
			// "
			//				+ file.getAbsolutePath()
			//				+ " generated and exception");
			return null;
		} finally {
			try {
				if (in != null) {
					in.close();
				}
			} catch (IOException ioe) {
				// 
			}
		}
		return identify(data);
	}



	private void init() {

		String formatString = "archive;BZ2;Bzip2 (bz2);application/x-bzip2;bz2,tbz2;0;425a683931;20;"
				+ "archive;GZ;GNU Zip (gz);application/x-gzip;gz,tgz;0;1f8b;20;"
				+ "archive;RAR;Rar;application/x-rar-compressed;rar;0;526172211a;20;"
				+ "archive;ZIP;PKWare Zip (ZIP);application/zip;zip;0;\"PK\";20;"
				+ "audio;MIDI;Midi (MID);audio/unknown;mid,midi;0;\"MThd\";20;"
				+ "audio;MP3;MPEG audio (mp3);audio/mpeg;mp3,mp2;0;\"ID3\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<HEAD\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<head\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<TITLE\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<title\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<!doctype HTML\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<!DOCTYPE HTML\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<!--\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<html\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<HTML\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<h1>\";20;"
				+ "doc;HTML;Hypertext Markup Language (HTML);text/html;htm,html;0;\"<H1>\";20;"
				+ "doc;XML;Extensible Markup Language (XML);text/xml;xml;0;\"<?xml\";10;"
				+ "doc;PDF1.3;Portable Document Format (PDF);application/pdf;pdf;0;\"%PDF-1.3\";20;"
				+ "doc;PDF1.4;Portable Document Format (PDF);application/pdf;pdf;0;\"%PDF-1.4\";20;"
				+ "doc;PDF1.5;Portable Document Format (PDF);application/pdf;pdf;0;\"%PDF-1.5\";20;"
				+ "doc;PS;PostScript (PS);application/postscript;ps;0;\"%!\";20;"
				+ "doc;RTF;Rich Text Format (RTF);text/rtf;rtf;0;\"{\rtf\";20;"
				+ "exec;EXE;DOS/Windows executable (EXE);application/x-dosexec;exe;0;\"MZ\";20;"
				+ "image;BMP;Windows Bitmap (BMP);image/bmp;bmp;0;\"BM\";40;"
				+ "image;GIF;Graphics Interchange Format (GIF);image/gif;gif;0;\"GIF8\";40;"
				+ "image;JPEG;Joint Photographic Experts Group (JPG);image/jpeg;jpg;0;ffd8ff;40;"
				+ "image;PBM;Portable Bitmap (PBM);image/x-portable-bitmap;pgm;0;\"P1\";40;"
				+ "image;PBM;Portable Bitmap (PBM);image/x-portable-bitmap;pgm;0;\"P4\";40;"
				+ "image;PGM;Portable Graymap (PGM);image/x-portable-greymap;pgm;0;\"P2\";40;"
				+ "image;PGM;Portable Graymap (PGM);image/x-portable-greymap;pgm;0;\"P5\";40;"
				+ "image;PNG;Portable Network Graphics (PNG);image/png;png;0;89504e470d0a1a;40;"
				+ "image;PPM;Portable Pixmap (PPM);image/x-portable-pixmap;ppm;0;\"P3\";40;"
				+ "image;PPM;Portable Pixmap (PPM);image/x-portable-pixmap;ppm;0;\"P6\";40;"
				+ "image;TIFF;Tag Image File Format (TIFF);image/tiff;tif;0;4d4d002a;40;"
				+ "image;TIFF;Tag Image File Format (TIFF);image/tiff;tif;0;49492a00;40;"
				+ "video;AVI;Audio Video Interleave (AVI);video/x-msvideo;avi;8;41564920;20;"
				+ "video;MOV;Quicktime video (MOV);video/quicktime;mov;4;\"moov\";20;"
				+ "video;MPEG;MPEG video (MPG);video/mpeg;mpg,mpeg;0;000001b3;20;"
				+ "video;MPEG;MPEG video (MPG);video/mpeg;mpg,mpeg;0;000001ba;20;";

		StringReader stringStream = new StringReader(formatString);

	}

	/**
	 * @param inStreamReader
	 * @throws IOException
	 */
	private void init(InputStreamReader inStreamReader) throws IOException {

		FormatDescriptionReader in = new FormatDescriptionReader(inStreamReader);
		FormatDescription desc;
		while ((desc = in.read()) != null) {
			//System.out
			//		.println("----------------------------- Format Description: fill
			// in descriptions: "
			//				+ desc.getShortName());
			byte[] magic = desc.getMagicBytes();
			Integer offset = desc.getOffset();
			if (magic != null && offset != null
					&& offset.intValue() + magic.length > minBufferSize) {
				minBufferSize = offset.intValue() + magic.length;
			}
			descriptions.add(desc);
		}
		inStreamReader.close();
	}

	/**
	 * Main method, to run directly the example. *
	 * 
	 * @param args
	 *            String[]
	 * @param args
	 *            String[]
	 */
	public static void main(String[] args) {

		Vector supportedFigureFormats = new Vector();
		supportedFigureFormats.add("JPG");
		supportedFigureFormats.add("JPEG");
		supportedFigureFormats.add("PNG");
		supportedFigureFormats.add("PDF1.0");
		supportedFigureFormats.add("PDF1.1");
		supportedFigureFormats.add("PDF1.2");
		supportedFigureFormats.add("PDF1.3");
		//supportedFigureFormats.add("PDF1.4");

//		//File imageFile = new File(
//		//		"J:/tmp/waspProposals/ctsj0103_1.pdf_1080641434502.tmp");
//		File imageFile = new File(
//				"J:/projects/phase1/esoform-74A/central.png");
//
		FileReader inStreamReader = null;
		try {
			inStreamReader = new FileReader("D:/TEMP/metadata/formats.txt");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		FormatIdentification identification = FormatIdentification
				.getInstance(inStreamReader);
//
//		
//		String shortDesc = "";
//		  FormatDescription desc = identification.identify(imageFile); 
//		  if (desc ==null) { 
//		  	System.out.println("Error getting description for file: " +
//		  imageFile.getAbsolutePath()); 
//		  } else { 
//		  	shortDesc =desc.getShortName(); 
//		  }
//		  
//		  
//		  if (supportedFigureFormats.contains(shortDesc) == false) { 
//		  	System.out.println("The format " + shortDesc +
//		  			" is not supported (file: " + imageFile.getAbsolutePath()); 
//		  	} else {
//		  System.out.println("The file " + imageFile.getName() + " has the format: " + 
//		  		shortDesc);
//		  		 }
		 
		File directory = new File("J:/tmp/P74_pdf");
		File[] fileList = directory.listFiles();
		for (int i = 0; i < fileList.length; i++) {

			//System.out.println("Analyzing file: "
			//		+ fileList[i].getAbsolutePath());
			FormatDescription desc = identification.identify(fileList[i]
					.getAbsoluteFile());

			if (desc == null) {
				System.out.println("Error getting description for file: "
						+ fileList[i].getAbsolutePath());
			} else {
				String shortDesc = desc.getShortName();
				//System.out.println("File " + fileList[i].getAbsolutePath()
				//		+ " has description:" + shortDesc);
				if (supportedFigureFormats.contains(shortDesc) == false) {
					System.out.println("The format " + shortDesc
							+ " is not supported (file: "
							+ fileList[i].getAbsolutePath());
				}
			}
		}

		System.out.println("Done");
	}
}

